#ifndef BatchLab_BaseTuple_h
#define BatchLab_BaseTuple_h

// Inheritance :
#include <BatchLab/Core/BaseObject.h>
#include <AIDA/ITuple.h>
#include <AIDA/ITupleEntry.h> //FIXME
#include <AIDA/IConstants.h> //For AIDA_VERSION
#if (AIDA_VERSION_MAJOR<=3) && (AIDA_VERSION_MINOR<=2) 
#else
#include <AIDA/ITupleColumn.h>
#endif

#include <Lib/Value.h>
#include <BatchLab/Core/Annotation.h>

#include <list>

namespace BatchLab {

class BaseColumn 
#if (AIDA_VERSION_MAJOR<=3) && (AIDA_VERSION_MINOR<=2) 
#else
:public virtual AIDA::IBaseTupleColumn
#endif
{
public: //AIDA::IBaseTupleColumn
  virtual void* cast(const std::string&) const;
  virtual std::string name() const;
  virtual std::string type() const;
  virtual double minimum() const;
  virtual double maximum() const;
  virtual double mean() const;
  virtual double rms() const;
public:
  virtual unsigned int size() const = 0;
  virtual void reset() = 0;
  virtual void add() = 0;
  virtual BaseColumn* copy() const = 0;
  virtual void resetFilled() = 0;
  virtual std::string stringDefaultValue() const = 0;
  virtual bool isDefaultDefault() const = 0;
  virtual bool append(void*) = 0;
  virtual bool value(int aIndex,Lib::Value& aValue) const = 0;
public:
  virtual bool read(Slash::Store::IVisitor&) = 0;
protected:
  BaseColumn(Slash::Core::IWriter&,const std::string&,const std::string&);
  BaseColumn(const BaseColumn&);
public:
  virtual ~BaseColumn();
  void setRow(int);
protected:
  Slash::Core::IWriter& fPrinter;
  std::string fLabel;
  std::string fType;
  int fRow;
};

class BaseTuple 
:public BaseObject
,public virtual AIDA::ITuple
,public virtual AIDA::ITupleEntry  //FIXME
{
public:
  virtual void* cast(const std::string&) const;
  virtual std::string type() const;
public: //AIDA/ITuple
  virtual std::string title() const;
  virtual bool setTitle(const std::string&);
  virtual AIDA::IAnnotation & annotation();
  virtual const AIDA::IAnnotation & annotation() const;
  virtual int findColumn(const std::string&) const;
  virtual void resetRow();
  virtual void reset();
  virtual void start();
  virtual bool skip(int);
  virtual bool next();
  virtual bool setRow(int);
  virtual int columns() const;
  virtual std::string columnName(int) const;
  virtual std::vector<std::string>  columnNames() const;
  virtual std::string columnType(int) const;
  virtual std::vector<std::string>  columnTypes() const;
  virtual bool project(AIDA::IHistogram1D&,
                       AIDA::IEvaluator&);
  virtual bool project(AIDA::IHistogram1D&,
                       AIDA::IEvaluator&,
                       AIDA::IFilter&);
  virtual bool project(AIDA::IHistogram1D&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&);
  virtual bool project(AIDA::IHistogram1D&,
                       AIDA::IEvaluator&,
                       AIDA::IFilter&,
                       AIDA::IEvaluator&);
  virtual bool project(AIDA::IHistogram2D&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&);
  virtual bool project(AIDA::IHistogram2D&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IFilter&);
  virtual bool project(AIDA::IHistogram2D&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&);
  virtual bool project(AIDA::IHistogram2D&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IFilter&,
                       AIDA::IEvaluator&);
  virtual bool project(AIDA::IHistogram3D&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&);
  virtual bool project(AIDA::IHistogram3D&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IFilter&);
  virtual bool project(AIDA::IHistogram3D&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&);
  virtual bool project(AIDA::IHistogram3D&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IFilter&,
                       AIDA::IEvaluator&);
  virtual bool project(AIDA::ICloud1D&,
                       AIDA::IEvaluator&);
  virtual bool project(AIDA::ICloud1D&,
                       AIDA::IEvaluator&,
                       AIDA::IFilter&);
  virtual bool project(AIDA::ICloud1D&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&);
  virtual bool project(AIDA::ICloud1D&,
                       AIDA::IEvaluator&,
                       AIDA::IFilter&,
                       AIDA::IEvaluator&);
  virtual bool project(AIDA::ICloud2D&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&);
  virtual bool project(AIDA::ICloud2D&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IFilter&);
  virtual bool project(AIDA::ICloud2D&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&);
  virtual bool project(AIDA::ICloud2D&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IFilter&,
                       AIDA::IEvaluator&);
  virtual bool project(AIDA::ICloud3D&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&);
  virtual bool project(AIDA::ICloud3D&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IFilter&);
  virtual bool project(AIDA::ICloud3D&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&);
  virtual bool project(AIDA::ICloud3D&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IFilter&,
                       AIDA::IEvaluator&);

  virtual bool project(AIDA::IProfile1D&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&);
  virtual bool project(AIDA::IProfile1D&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IFilter&);
  virtual bool project(AIDA::IProfile1D&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&);
  virtual bool project(AIDA::IProfile1D&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IFilter&,
                       AIDA::IEvaluator&);
  virtual bool project(AIDA::IProfile2D&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&);
  virtual bool project(AIDA::IProfile2D&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IFilter&);
  virtual bool project(AIDA::IProfile2D&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&);
  virtual bool project(AIDA::IProfile2D&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IFilter&,
                       AIDA::IEvaluator&);
  virtual std::string columnDefaultValue(int) const;
  virtual double evaluateMin(AIDA::IEvaluator&); //AIDA-v3r3p0
  virtual double evaluateMin(AIDA::IEvaluator&,AIDA::IFilter&); //AIDA-v3r3p0
  virtual double evaluateMax(AIDA::IEvaluator&); //AIDA-v3r3p0
  virtual double evaluateMax(AIDA::IEvaluator&,AIDA::IFilter&); //AIDA-v3r3p0
#if (AIDA_VERSION_MAJOR<=3) && (AIDA_VERSION_MINOR<=2) 
  virtual unsigned char getByte(int) const = 0;
#else
  virtual AIDA::IBaseTupleColumn* column(const std::string&) const;
  virtual AIDA::IBaseTupleColumn* column(int) const;
#endif
public:
  virtual bool isDefaultDefault(int) const;
  virtual int rowIndex() const;
  virtual bool close();
public:
  BaseColumn* findColumn1(int) const; //FIXME : protected. Used in HDF5.
protected:
  BaseTuple(Slash::Core::IWriter&,const std::string&,const std::string&);
  BaseTuple(const BaseTuple&);
  virtual ~BaseTuple();
  Annotation& getAnnotation(); //annotation() is ambiguous vs the AIDA one.
  void setAnnotation(const Annotation&);
protected:
  static bool booking(Slash::Core::IWriter&,const std::string&,
                      std::list<Lib::Value>&);
protected:
  void clear();
  std::string convertType(Slash::Core::IValue::Type) const; 
  void setColumnsRow(int); 
protected:
  Annotation fAnnotation;
  std::string fTitle;
  int fRow;
  std::vector<BaseColumn*> fColumns;
};

}

#endif
