//
//  Due to linking to F77 COMMONS, we can't have
// the below callbacks in a G3Lab DLL used by other
// applications like G3Simple.
//  The below callbacks have to be embeded in the
// other G3 applications callbacks DLLS. It is why
// they are put in a .h with some Xxx_ prefix. 
// Someone has then to include this file and redefine
// (with cpp macros) the callbacks to local names.
//  See G3Simple/<version>/source/Callbacks.cxx for
// an example.
//


// G3Lab :
#include <G3Lab/gmanip.h>
#include <G3Lab/SoF77.h>
#include <G3Lab/Types.h>

#include <Slash/Tools/cast.h>

inline void init_VOLU_ids(Slash::Core::ISession& aSession){
  Slash::Data::IProcessor* da = Slash::accessorManager(aSession);
  if(da) {
    Slash::Data::IAccessor* acc = da->findAccessor("VOLU");
    if(acc) {
      G3Lab::VOLU_Type* type = Slash_cast(*acc,G3Lab::VOLU_Type);
      type->fill_ids();
    }
  }
}

inline void init_EVENT_ids(Slash::Core::ISession& aSession,bool a_do_digit = false){
  Slash::Data::IProcessor* da = Slash::accessorManager(aSession);
  if(da) {
   {Slash::Data::IAccessor* acc = da->findAccessor("VERT");
    if(acc) {
      G3Lab::VERT_Type* type = Slash_cast(*acc,G3Lab::VERT_Type);
      type->fill_ids();
    }}
   {Slash::Data::IAccessor* acc = da->findAccessor("KINE");
    if(acc) {
      G3Lab::KINE_Type* type = Slash_cast(*acc,G3Lab::KINE_Type);
      type->fill_ids();
    }}

    if(a_do_digit) {
      Slash::Data::IAccessor* acc = da->findAccessor("DIGI");
      if(acc) {
        G3Lab::DIGI_Type* type = Slash_cast(*acc,G3Lab::DIGI_Type);
        type->fill_ids();
      }
    }
  }
}

extern "C" {

void G3Lab_initialize(Slash::UI::IUI& aUI,const std::vector<std::string>& aArgs) {
  // Should be executed (in a widget create callback) 
  // when the GUI is constructed 
  Slash::Core::ISession& session = aUI.session();
  Slash::Data::IProcessor* da = Slash::accessorManager(session);
  if(da && !da->findAccessor("VOLU")) {
    da->addAccessor(new G3Lab::VOLU_Type(session));
    da->addAccessor(new G3Lab::VERT_Type(session));
    da->addAccessor(new G3Lab::KINE_Type(session));
    da->addAccessor(new G3Lab::DIGI_Type(session));
  }
  SoF77_SetSession(&session);
  if(aArgs.size()&&(aArgs[0]=="no_geant3_init")) return;
  G3Lab::gmanip::initialize();
}

void G3Lab_load_geometry(Slash::UI::IUI& aUI,const std::vector<std::string>& aArgs) {
  if(!ui_check_args(aUI,aArgs,1)) return;
  int unit = 70;
  G3Lab::gmanip::loadGeometry(21,aArgs[0],unit);
  init_VOLU_ids(aUI.session());
}

void G3Lab_print_volumes(Slash::UI::IUI&) {G3Lab::gmanip::printVolumes();}

void G3Lab_next_event(Slash::UI::IUI& aUI) {
  int unit = 70;
  G3Lab::gmanip::readEvent(unit);
  init_EVENT_ids(aUI.session());
}

void G3Lab_set_tree(Slash::UI::IUI& aUI,const std::vector<std::string>& aArgs){
  // args[0] widget tree.
  // args[1] [volume_names]
  if(!ui_check_args(aUI,aArgs,2)) return;
  if(aArgs[1]=="volume_names") {
    std::vector<std::string> vec = G3Lab::gmanip::volumeNames();
    std::string sout = "<tree>";
    for(unsigned int index=0;index<vec.size();index++) {
      sout += "<treeItem><label>"+vec[index]+"</label></treeItem>";
    }
    sout += "</tree>";
    aUI.setParameter(aArgs[0],sout);
  }
}

} // extern "C"
