
#include <Slash/Tools/system.h>
#include <Slash/Tools/smanip.h>
#include <Slash/Tools/args.h>
#include <Slash/Tools/file.h>
#include <Slash/Tools/fmanip.h>
#include <Slash/Tools/directory.h>

// Embedded code :
#include "ui_onx_cxx.h"
#include "session_start_cxx.h"

inline bool file_replace(const std::string& a_file,const std::string& a_old,const std::string& a_new,std::vector<std::string>& a_changes){
  a_changes.clear();
  std::vector<std::string> text;
  if(!Slash::file::read(a_file,text)) return false;
  // Replace :
  bool touched = false;
 {std::vector<std::string>::iterator it;
  for(it=text.begin();it!=text.end();++it) {
    std::string old = *it;
    if(Slash::replace(*it,a_old,a_new)) {
      a_changes.push_back(old);
      a_changes.push_back(*it);
      touched = true;
    }
  }}
  if(!touched) return true;
  // Create temporary file :
  //NOTE : have a tmpName in current directory so that the below 
  //       rename works. (Else we should do a move).
  std::string tmp_name;
  if(!Slash::tmpname(".","Lib_","",tmp_name)) return false;
  FILE* file = ::fopen(tmp_name.c_str(),"wb");
  if(!file) return false;
 {unsigned int textn = text.size();
  for(unsigned index=0;index<textn;index++) {
    if(::fprintf(file,"%s\n",text[index].c_str())<0) {
      ::fclose(file);
      Slash::file::remove(tmp_name);
      return false;
    }
  }}
  ::fclose(file);
  if(!Slash::file::remove(a_file)) return false;
  if(!Slash::file::rename(tmp_name,a_file)) return false;
  return true;
}

#include <cstdlib>

#ifdef PROGRAM_NAME
#define S(aWhat) #aWhat
#define XS(aWhat) S(aWhat)
#define PROGRAM__NAME XS(PROGRAM_NAME)
#else
#define PROGRAM__NAME "G4Lab_g4_to_osc"
#endif

static bool find_main(const std::string& aLine,bool& a_argc) {
  // Looking for "main" followed by a "(" and then a ")"
  std::string::size_type pos = aLine.find("main");
  if(pos!=std::string::npos) {
    std::string::size_type pos_main = pos;
    pos = aLine.find('(',pos+4);
    if(pos!=std::string::npos) {
      a_argc = aLine.find("argc",pos+1)?true:false;
      pos = aLine.find(')',pos+1);
      if(pos!=std::string::npos) {
        //Look if not a main() in a comment ! (See N04 !)
        pos = aLine.find("//");
        if( (pos!=std::string::npos) && (pos<pos_main)) {
          return false;
        }
        return true;
      }
    }
  }
  return false;
}

static bool find_G4RunManager(const std::string& aLine,std::string& aVariable){
  // Looking for :  G4RunManager* runManager = new G4RunManager;

  std::string::size_type pos = aLine.find("G4RunManager()");
  if(pos==std::string::npos) {
    pos = aLine.find("G4RunManager;");
  }

  if(pos!=std::string::npos) {
    std::string beg = aLine.substr(0,pos);
    pos = beg.find("new");
    if(pos!=std::string::npos) {
      beg = beg.substr(0,pos);
      pos = beg.find('=');
      if(pos!=std::string::npos) {
        beg = beg.substr(0,pos);
        Slash::strip(beg);
        pos = beg.rfind(' ');
        if(pos!=std::string::npos) {
          aVariable = beg.substr(pos+1,beg.size()-pos);
        } else {
          aVariable = beg;
        }
        Slash::strip(aVariable);
        if(aVariable.size()) {
          return true;
        }
      }
    }
  }

  return false;
}

static bool find_ROOT(const std::string& aLine) {
  if(aLine.find("TTree")!=std::string::npos) return true;
  if(aLine.find("TFile")!=std::string::npos) return true;
  if(aLine.find("TH1")!=std::string::npos) return true;
  if(aLine.find("TH2")!=std::string::npos) return true;
  return false; // Ouf !
}

static bool find_G4UIsession(const std::string& aLine) {
  // Looking for :  G4UIsession * session = 0;
  std::string::size_type pos = aLine.find("G4UIsession");
  if(pos!=std::string::npos) {
    pos = aLine.find("*",pos+11);
    if(pos!=std::string::npos) {
      pos = aLine.find("session",pos+1);
      if(pos!=std::string::npos) {
        pos = aLine.find("=",pos+7);
        if(pos!=std::string::npos) {
          pos = aLine.find("0",pos+1);
          if(pos!=std::string::npos) {
            return true;
          }
        }
      }
    }
  }
  return false;
}

static bool find_exec_vis_mac(const std::string& aLine) {
  // Looking for : UI->ApplyCommand("/control/execute vis.mac");     
  std::string::size_type pos = aLine.find("ApplyCommand");
  if(pos!=std::string::npos) {
    pos = aLine.find("execute",pos+12);
    if(pos!=std::string::npos) {
      pos = aLine.find("vis.mac",pos+7);
      if(pos!=std::string::npos) {
        return true;
      }
    }
  }
  return false;
}

static bool find_delete_session(const std::string& aLine) {
  // Looking for :  delete session;
  std::string::size_type pos = aLine.find("delete");
  if(pos!=std::string::npos) {
    pos = aLine.find("session",pos+6);
    if(pos!=std::string::npos) {
      return true;
    }
  }
  return false;
}

static bool find_delete_runManager(const std::string& aLine,const std::string& aVariable) {
  // Looking for :  delete runManager;
  std::string::size_type pos = aLine.find("delete");
  if(pos!=std::string::npos) {
    pos = aLine.find(aVariable,pos+6);
    if(pos!=std::string::npos) {
      return true;
    }
  }
  return false;
}

static bool find_G4UIterminal(const std::string& aLine) {
  // Looking for :  new G4UIterminal
  std::string::size_type pos = aLine.find("new");
  if(pos!=std::string::npos) {
    pos = aLine.find("G4UIterminal",pos+4);
    if(pos!=std::string::npos) {
      //pos = aLine.find("G4UItcsh",pos+12);
      //if(pos!=std::string::npos) return false;
      return true;
    }
  }
  return false;
}

static bool find_SessionStart(const std::string& aLine) {
  std::string::size_type pos = aLine.find("SessionStart()");
  return (pos!=std::string::npos?true:false);
}

static bool find_G4UI_USE(const std::string& aLine) {
  std::string s = aLine;
  Slash::strip(s);
  std::string::size_type pos = s.find("#ifdef G4UI_USE");
  return (pos!=std::string::npos?true:false);
}

static bool find_include_am(const std::string& aLine,const std::string& aClass) {
  // Looking for :  #include of <aClass> header
  std::string::size_type pos = aLine.find("#include");
  if(pos!=std::string::npos) {
    pos = aLine.find(aClass+".",pos+8);
    if(pos!=std::string::npos) {
      return true;
    }
    pos = aLine.find(aClass+">",pos+8);
    if(pos!=std::string::npos) {
      return true;
    }
  }
  return false;
}

static bool find_class_am(const std::string& aLine,const std::string& aClass) {
  // Looking for :  class <aClass>
  std::string::size_type pos = aLine.find("class");
  if(pos!=std::string::npos) {
    pos = aLine.find(aClass,pos+5);
    if(pos!=std::string::npos) {
      return true;
    }
  }
  return false;
}
static bool find_dstor_am(const std::string& aLine,const std::string& aClass) {
  // Looking for :  <aClass>
  std::string::size_type pos = aLine.find("~"+aClass);
  if(pos!=std::string::npos) {
    return true;
  }
  return false;
}
static bool find_AIDA_createAnalysisFactory(const std::string& aLine,std::string& aVariable) {
  // Looking for :  <variable> = AIDA_createAnalysisFactory();
  // We need the name of the variable to deconnect its delete.
  std::string::size_type pos = aLine.find("AIDA_createAnalysisFactory()");
  if(pos!=std::string::npos) {
    std::string beg = aLine.substr(0,pos);
    pos = beg.find('=');
    if(pos!=std::string::npos) {
      beg = beg.substr(0,pos);
      Slash::strip(beg);
      pos = beg.rfind(' ');
      if(pos!=std::string::npos) {
        beg = beg.substr(pos+1,beg.size()-pos);
      }
      Slash::strip(beg);
      //case of cosmicray_charging/LISAAnalysisManager.cc :
      //  if( (af = AIDA_createAnalysisFactory()) ) 
      pos = beg.rfind('(');
      if(pos!=std::string::npos) {
        aVariable = beg.substr(pos+1,beg.size()-pos);
      } else {
        aVariable = beg;
      }
      Slash::strip(beg);
      if(aVariable.size()) {
        return true;
      }
    }
  }
  return false;
}
static bool find_delete_variable(const std::string& aLine,const std::string& aVariable) {
  // Looking for :  delete <variable>;
  std::string::size_type pos = aLine.find("delete");
  if(pos!=std::string::npos) {
    pos = aLine.find(aVariable,pos+6);
    if(pos!=std::string::npos) {
      return true;
    }
  }
  return false;
}

static bool find_am_cc(std::string& aFile,std::ostream& a_out) {
  // Search in the src directory a source file 
  // containing a AIDA_createAnalysisFactory()
  if(!Slash::directory::cd("src")) {
    a_out << PROGRAM__NAME << " :"
          << " can't go in src directory."
          << std::endl;
    return false;
  }
  std::vector<std::string> files;
  if(!Slash::directory::entries(".",files,false)) {
    a_out << PROGRAM__NAME << " :"
          << " can't get file in src directory."
          << std::endl;
    return false;
  }
  for(std::vector<std::string>::const_iterator it=files.begin();
    it!=files.end();++it) {
    const std::string& s = *it;
    std::vector<std::string> v;
    if(!Slash::file::found(s,"AIDA_createAnalysisFactory()",v)) {}
    if(v.size()) {
      aFile = s;
      break;
    }
  }
  if(!Slash::directory::cd("..")) {
    a_out << PROGRAM__NAME << " :"
          << " can't go in .. directory."
          << std::endl;
    return false;
  }
  return true;
}

static bool find_am_hh(const std::string& aClass,std::string& aFile,std::ostream& a_out) {
  // Search in the include directory the <aClass> header file.
  if(!Slash::directory::cd("include")) {
    a_out << PROGRAM__NAME << " :"
          << " can't go in include directory."
          << std::endl;
    return false;
  }
  std::vector<std::string> files;
  if(!Slash::directory::entries(".",files,false)) {
    a_out << PROGRAM__NAME << " :"
          << " can't get file in src directory."
          << std::endl;
    return false;
  }
  for(std::vector<std::string>::const_iterator it=files.begin();
    it!=files.end();++it) {
    const std::string& s = *it;
    if(s.substr(0,aClass.size())==aClass) {
      aFile = s;
      break;
    }
  }
  if(!Slash::directory::cd("..")) {
    a_out << PROGRAM__NAME << " :"
          << " can't go in .. directory."
          << std::endl;
    return false;
  }
  return true;
}

bool find_SessionStart(const std::string& aFile,bool& a_value,std::ostream& a_out) {
  // Look for a line with SessionStart() :
  std::vector<std::string> text;
  if(!Slash::file::read(aFile,text)) {
    a_out << PROGRAM__NAME << " :"
          << " can't read " << Slash::sout(aFile) << "." 
          << std::endl;
    return false;
  }
  a_value = false;
  for(std::vector<std::string>::const_iterator it=text.begin();
      it!=text.end();++it) {
    if((*it).find("SessionStart()")!=std::string::npos) {
      a_value = true;
      break;
    }
  }
  return true;
}

bool treat_main(
 const std::string& aPackage
,const std::string& aFile
,bool a_aida_vis
,const std::string& a_am_hh
,const std::string& a_am_class
,bool a_verbose,std::ostream& a_out) {

  std::vector<std::string> text;
  if(!Slash::file::read(aFile,text)) {
    a_out << PROGRAM__NAME << " :"
          << " can't read " << Slash::sout(aFile) << "." 
          << std::endl;
    return false;
  }

  std::vector<std::string> new_text;

  std::string run_manager_variable;

 {bool main_found = false;
  bool G4UI_USE_found = false;
  bool session_start_found = false;
  bool run_found = false;
  bool have_argc = false;
  bool delete_session_found = false;
  bool delete_runManager_found = false;
  bool ROOT_found = false;
  for(std::vector<std::string>::const_iterator it=text.begin();
      it!=text.end();++it) {
    if((*it).find("<G4Lab/UIOnX.h>")!=std::string::npos) {
      //already done.
      a_out << PROGRAM__NAME << " :"
            << " file " << Slash::sout(aFile) 
            << " already treated."
            << std::endl;
      return false;  
    }

    if(find_main(*it,have_argc)) {
      new_text.push_back("//OSC:begin");
      new_text.push_back("#include <Slash/Tools/app.h>");
      new_text.push_back("#include <Slash/Tools/osc_env.h>");
      if(aPackage=="TestEm1") {
        new_text.push_back("#include <Slash/Tools/file.h>");
      }
      new_text.push_back("#include <cstdlib>");
      new_text.push_back("#include <string>");
      new_text.push_back("#include <iostream>");
      new_text.push_back("#include <G4Lab/UIOnX.h>");
      if(a_aida_vis) {
        new_text.push_back("#include \""+a_am_hh+"\"");
        if(have_argc) {
          new_text.push_back("#include <AIDA/IAnalysisFactory.h>");
        }
      }
      new_text.push_back("//OSC:end");
      new_text.push_back("");
      new_text.push_back(*it);

      bool add_psn = false;
      if(have_argc) {
        std::string::size_type pos = (*it).find('{');
        if(pos!=std::string::npos) {
	  add_psn = true;
        } else {
          // the { on next line ?
          it++;
          if(it==text.end()) break;
          new_text.push_back(*it);
          std::string::size_type pos = (*it).find('{');
          if(pos!=std::string::npos) {
  	    add_psn = true;
          }
        }
      }

      if(add_psn) {
        new_text.push_back("//OSC:begin");
        new_text.push_back("#ifdef __APPLE__");
        new_text.push_back("  bool is_mac_app = Slash::argcv_is_mac_app(argc,argv);");
	new_text.push_back("  if(is_mac_app) argc = 1;");
	new_text.push_back("#endif");
 	new_text.push_back("//OSC:end");
      }

      main_found = true;
    } else if(find_G4RunManager(*it,run_manager_variable)) {
      if(a_verbose) {
        a_out << " The run manager variable is " 
              << Slash::sout(run_manager_variable) << "."
              << std::endl;
      }
      new_text.push_back(*it);
      if(a_aida_vis) {
        new_text.push_back("");
        new_text.push_back("//OSC:begin");
        new_text.push_back("  G4UIsession* session = 0;");
        if(have_argc) {
          new_text.push_back("  if (argc!=1) {   // batch mode");
          new_text.push_back("  "+a_am_class+"::setAIDA(AIDA_createAnalysisFactory());");
          new_text.push_back("  } else {");
        } else {
          new_text.push_back("  //if (argc!=1) {   // batch mode");
          new_text.push_back("  //"+a_am_class+"::setAIDA(AIDA_createAnalysisFactory());");
          new_text.push_back("  //} else {");
        }
        if(run_manager_variable.empty()) {
          a_out << PROGRAM__NAME << " :"
                << " no run manager variable found (1)."
                << std::endl;
          return false;
        }

       {for(unsigned int index=0;index<ui_onx_cxx_linen;index++) {
          std::string s = ui_onx_cxx_lines[index];
          Slash::replace(s,"@@double_quote@@","\"");
          Slash::replace(s,"@@back_slash@@","\\");
          Slash::replace(s,"@@package@@",aPackage);
          Slash::replace(s,"@@run_manager_variable@@",run_manager_variable);
          new_text.push_back(s);
        }}

        new_text.push_back("  // Enforce the creation of the GUI here, else");
        new_text.push_back("  // the AIDA plotter factory will create its own");
        new_text.push_back("  // that will interfer with the G4Lab::UIOnX one.");
        new_text.push_back("  uiOnX->createUI();");
        new_text.push_back("  "+a_am_class+"::setAIDA(uiOnX->findAIDA());");
      //new_text.push_back("  session = uiOnX;");
        if(have_argc) {
          new_text.push_back("  }");
        } else {
          new_text.push_back("  //}");
        }
        new_text.push_back("//OSC:end");
      }
      run_found = true;
    } else if(find_G4UIsession(*it)) {
      new_text.push_back("//OSC:begin");
      new_text.push_back("//"+(*it));
      new_text.push_back("//OSC:end");
    } else if(find_exec_vis_mac(*it)) {
      new_text.push_back("//OSC:begin");
      new_text.push_back("//"+(*it));
      new_text.push_back("//OSC:end");
    } else if(find_G4UIterminal(*it)) {
      new_text.push_back("//OSC:begin");
      new_text.push_back("//"+(*it));
      new_text.push_back("//OSC:end");
    } else if(a_aida_vis && find_delete_session(*it)) {
      new_text.push_back("//OSC:begin");
      new_text.push_back("//"+(*it));
      new_text.push_back("//OSC:end");
      delete_session_found = true;
    } else if(a_aida_vis && find_delete_runManager(*it,run_manager_variable)) {
      new_text.push_back(*it);
      new_text.push_back("");
      new_text.push_back("//OSC:begin");
      new_text.push_back("  // The delete of the run manager may induce,");
      new_text.push_back("  // through the user action classes,");
      new_text.push_back("  // a delete / closing of the AIDA resources");
      new_text.push_back("  // as AIDA trees and various factories.");
      new_text.push_back("  // Then we delete the UIOnX that manages the");
      new_text.push_back("  // AIDA implementation after the delete");
      new_text.push_back("  // of the G4 run manager.");
      new_text.push_back("  delete session;");
      new_text.push_back("//OSC:end");
      delete_runManager_found = true;

    } else if(main_found && a_aida_vis && find_G4UI_USE(*it)) {
      //G4UIExecutive way of doing.
      new_text.push_back("//OSC:begin");
      new_text.push_back("      session->SessionStart();");
      new_text.push_back("//OSC:end");
      new_text.push_back(*it);
      G4UI_USE_found = true;

    } else if(main_found && !a_aida_vis && find_G4UI_USE(*it)) {
      //G4UIExecutive way of doing.

      if(run_manager_variable.empty()) {
        a_out << PROGRAM__NAME << " :"
              << " no run manager variable found (2)."
              << std::endl;
        return false;
      }
      new_text.push_back("//OSC:begin");

      new_text.push_back("      G4UIsession* session = 0;");
     {for(unsigned int index=0;index<ui_onx_cxx_linen;index++) {
        std::string s = ui_onx_cxx_lines[index];
        Slash::replace(s,"@@double_quote@@","\"");
        Slash::replace(s,"@@back_slash@@","\\");
        Slash::replace(s,"@@package@@",aPackage);
        Slash::replace(s,"@@run_manager_variable@@",run_manager_variable);
        new_text.push_back(s);
      }}

      new_text.push_back("      session->SessionStart();");
      new_text.push_back("      delete session;");

      new_text.push_back("//OSC:end");
      new_text.push_back(*it);
      G4UI_USE_found = true;

    } else if(!G4UI_USE_found && !a_aida_vis && find_SessionStart(*it)) {
      //before G4UIExecutive way of doing.

      if(run_manager_variable.empty()) {
        a_out << PROGRAM__NAME << " :"
              << " no run manager variable found (2)."
              << std::endl;
        return false;
      }
      new_text.push_back("//OSC:begin");

      new_text.push_back("      G4UIsession* session = 0;");
     {for(unsigned int index=0;index<ui_onx_cxx_linen;index++) {
        std::string s = ui_onx_cxx_lines[index];
        Slash::replace(s,"@@double_quote@@","\"");
        Slash::replace(s,"@@back_slash@@","\\");
        Slash::replace(s,"@@package@@",aPackage);
        Slash::replace(s,"@@run_manager_variable@@",run_manager_variable);
        new_text.push_back(s);
      }}

      new_text.push_back("//OSC:end");
      new_text.push_back(*it);
      session_start_found = true;
    } else if(find_ROOT(*it)) {
      new_text.push_back(*it);
      ROOT_found = true;
    } else {
      new_text.push_back(*it);
    }
  }
  if(!main_found) {
    a_out << PROGRAM__NAME << " :"
          << " main function not found in file " 
          << Slash::sout(aFile) << "."
          << std::endl;
    return false;
  }
  if(ROOT_found) {
//        << " You have to arrange to be able to disconnect \"that\"." << std::endl
//        << " For example by bracketing ROOT code within" << std::endl
//        << " some #ifdef G4USE_ROOT as for other external drivers / tools."
//        << std::endl
    a_out << PROGRAM__NAME << " :"
          << " WARNING : this code uses ROOT :-( :-( :-(."
          << " Continue anyway."
          << std::endl;
  }
  if(!a_aida_vis && !session_start_found && !G4UI_USE_found) {
    a_out << PROGRAM__NAME << " :"
      //          << " line with \"SessionStart()\" not found in file " 
          << " line with \"SessionStart()\" or G4UI_USE not found"
          << " in main in  " << Slash::sout(aFile) << "."
          << std::endl;
    return false;
  }
  if(a_aida_vis && !run_found) {
    a_out << PROGRAM__NAME << " :"
          << " line with \"new G4RunManager\" not found in file " 
          << Slash::sout(aFile) << "."
          << std::endl;
    return false;
  }
  if(a_aida_vis && !delete_session_found && !G4UI_USE_found) {
    a_out << PROGRAM__NAME << " :"
          << " line with \"delete sesssion;\" not found in file " 
          << Slash::sout(aFile) << "."
          << std::endl;
    return false;
  }
  if(a_aida_vis && !delete_runManager_found) {
    a_out << PROGRAM__NAME << " :"
          << " line with \"delete runManager;\" not found in file " 
          << Slash::sout(aFile) << "."
          << std::endl;
    return false;
  }}

  if(!Slash::file::copy(aFile,aFile+"_before_osc")) {
    a_out << PROGRAM__NAME << " :"
          << " can't copy "
          << Slash::sout(aFile) << " to "
          << Slash::sout(aFile+"_before_osc") << "."
          << std::endl;
    return false;
  }

  if(!Slash::file::write(aFile,new_text)) {
    a_out << PROGRAM__NAME << " :"
          << " can't write " 
          << Slash::sout(aFile) << "."
          << std::endl;
    return false;
  }

  if(a_verbose) {
    a_out << " " << Slash::sout(aFile) << " modified." 
          << " Original file copied to " << aFile << "_before_osc."
          << std::endl;
  }

  return true;
}

bool treat_am_cc(const std::string& aFile,const std::string& aClass,bool a_verbose,std::ostream& a_out) {

  if(!Slash::directory::cd("src")) {
    a_out << PROGRAM__NAME << " :"
          << " can't go in src directory."
          << std::endl;
    return false;
  }

  std::vector<std::string> text;
  if(!Slash::file::read(aFile,text)) {
    a_out << PROGRAM__NAME << " :"
          << " can't read " << Slash::sout(aFile) << "." 
          << std::endl;
    return false;
  }

  std::vector<std::string> new_text;

  bool include_h_found = false;
  std::string variable;
 {for(std::vector<std::string>::const_iterator it=text.begin();
      it!=text.end();++it) {
    const std::string& line = (*it);
    if(find_include_am(line,aClass)) {
      new_text.push_back(*it);
      new_text.push_back("");
      new_text.push_back("//OSC:begin");
      //new_text.push_back("namespace AIDA {class IAnalysisFactory;}");
      new_text.push_back("AIDA::IAnalysisFactory* "+aClass+"::m_AIDA = 0;");
      new_text.push_back("void "+aClass+"::setAIDA(AIDA::IAnalysisFactory* a_AIDA) {m_AIDA = a_AIDA;}");
      new_text.push_back("AIDA::IAnalysisFactory* "+aClass+"::getAIDA(){return m_AIDA;}");
      new_text.push_back("//OSC:end");
      new_text.push_back("");

      include_h_found = true;
    } else if(find_AIDA_createAnalysisFactory(line,variable)) {
      std::string s = line;
      Slash::replace(s,"AIDA_createAnalysisFactory","getAIDA");
      new_text.push_back("//OSC:begin");
      new_text.push_back(s);
      new_text.push_back("//OSC:end");
    } else {
      new_text.push_back(*it);
    }
  }}
  if(!include_h_found) {
    a_out << PROGRAM__NAME << " :"
          << " #include of " << aClass 
          << " header not found in file "
          << Slash::sout(aFile) << "."
          << std::endl;
    return false;
  }

  if(variable.empty()) {
    a_out << PROGRAM__NAME << " :"
          << " AIDA::IAnalysisFactory variable"
          << " not found in file "
          << Slash::sout(aFile) << "."
          << std::endl;
    return false;
  } else {
    text = new_text;
    new_text.clear();
    bool delete_variable_found = false;
    for(std::vector<std::string>::const_iterator it=text.begin();
        it!=text.end();++it) {
      const std::string& line = (*it);
      if(find_delete_variable(line,variable)) {
        if(a_verbose) {
          a_out << " Assume that the variable handling the"
                << " AIDA::IAnalysisFactory pointer in the"
                << " analysis manager is " << Slash::sout(variable) << "."
                << std::endl;
        }
        new_text.push_back("//OSC:begin");
        new_text.push_back("// AIDA analysis factory is managed by");
        new_text.push_back("// G4Lab::UIOnX, it can't be delete here.");
        new_text.push_back("//"+(*it));
        new_text.push_back("//OSC:end");
        delete_variable_found = true;
      } else {
        new_text.push_back(*it);
      }
    }
    if(!delete_variable_found) {
      a_out << PROGRAM__NAME << " :"
            << " delete of variable " << variable
            << " not found in file "
            << Slash::sout(aFile) << "."
            << std::endl;
      return false;
    }
  }
 
  if(!Slash::file::copy(aFile,aFile+"_before_osc")) {
    a_out << PROGRAM__NAME << " :"
          << " can't copy "
          << Slash::sout(aFile) << " to "
          << Slash::sout(aFile+"_before_osc") << "."
          << std::endl;
    return false;
  }

  if(!Slash::file::write(aFile,new_text)) {
    a_out << PROGRAM__NAME << " :"
          << " can't write " 
          << Slash::sout(aFile) << "."
          << std::endl;
    return false;
  }

  if(!Slash::directory::cd("..")) {
    a_out << PROGRAM__NAME << " :"
          << " can't go in .. directory."
          << std::endl;
    return false;
  }

  if(a_verbose) {
    a_out << " " << Slash::sout(aFile) << " modified." 
          << " Original file copied to " << aFile << "_before_osc."
          << std::endl;
  }

  return true;
}

bool treat_am_hh(const std::string& aFile,const std::string& aClass,bool a_verbose,std::ostream& a_out) {

  if(!Slash::directory::cd("include")) {
    a_out << PROGRAM__NAME << " :"
          << " can't go in include directory."
          << std::endl;
    return false;
  }

  std::vector<std::string> text;
  if(!Slash::file::read(aFile,text)) {
    a_out << PROGRAM__NAME << " :"
          << " can't read " << Slash::sout(aFile) << "." 
          << std::endl;
    return false;
  }

  std::vector<std::string> new_text;

  bool cstor_found = false;
 {bool class_found = false;
  for(std::vector<std::string>::const_iterator it=text.begin();
      it!=text.end();++it) {
    const std::string& line = (*it);
    if(!class_found && find_class_am(line,aClass)) {
      new_text.push_back(*it);
      class_found = true;
      continue;
    }
    if(class_found && find_dstor_am(line,aClass)) {
      new_text.push_back("");
      new_text.push_back("//OSC:begin");
      new_text.push_back("public:");
      new_text.push_back("  static AIDA::IAnalysisFactory* m_AIDA;");
      new_text.push_back("  static void setAIDA(AIDA::IAnalysisFactory*);");
      new_text.push_back("  static AIDA::IAnalysisFactory* getAIDA();");
      new_text.push_back("public:");
      new_text.push_back("//OSC:begin");
      new_text.push_back("");

      new_text.push_back(*it);
      cstor_found = true;
    } else {
      new_text.push_back(*it);
    }
  }}
  if(!cstor_found) {
    a_out << PROGRAM__NAME << " :"
          << " constructor of " << aClass 
          << " not found in header file "
          << Slash::sout(aFile) << "."
          << std::endl;
    return false;
  }

  if(!Slash::file::copy(aFile,aFile+"_before_osc")) {
    a_out << PROGRAM__NAME << " :"
          << " can't copy "
          << Slash::sout(aFile) << " to "
          << Slash::sout(aFile+"_before_osc") << "."
          << std::endl;
    return false;
  }

  if(!Slash::file::write(aFile,new_text)) {
    a_out << PROGRAM__NAME << " :"
          << " can't write " 
          << Slash::sout(aFile) << "."
          << std::endl;
    return false;
  }

  if(!Slash::directory::cd("..")) {
    a_out << PROGRAM__NAME << " :"
          << " can't go in .. directory."
          << std::endl;
    return false;
  }

  if(a_verbose) {
    a_out << " " << Slash::sout(aFile) << " modified." 
          << " Original file copied to " << aFile << "_before_osc."
          << std::endl;
  }

  return true;
}

bool find_name_in_GNUmakefile(std::string& a_name,bool a_verbose,std::ostream& a_out) {

  std::vector<std::string> text;
  if(!Slash::file::read("GNUmakefile",text)) {
    a_out << PROGRAM__NAME << " :"
          << " can't read GNUmakefile." 
          << std::endl;
    return false;
  }

  bool name_found = false;
  for(std::vector<std::string>::const_iterator it=text.begin();
      it!=text.end();++it) {
    const std::string& line = (*it);
    if(line.substr(0,5)!="name ") continue;
    std::string::size_type pos = line.rfind(' ');
    if(pos!=std::string::npos) {
      a_name = line.substr(pos+1,line.size()-pos);
      Slash::strip(a_name);
      Slash::strip(a_name,Slash::trailing,'#'); //exMPI01,02
      name_found = true;
      break;
    }
  }
  if(!name_found) {
    a_out << PROGRAM__NAME << " :"
          << " name not found in GNUmakefile." 
          << std::endl;
    return false;
  }

  if(a_verbose) {
    a_out << " name in GNUmakefile is " << Slash::sout(a_name) << "."
          << std::endl;
  }

  return true;
}

bool treat_GNUmakefile(bool a_aida_vis,bool a_verbose,std::ostream& a_out) {

  std::vector<std::string> text;
  if(!Slash::file::read("GNUmakefile",text)) {
    a_out << PROGRAM__NAME << " :"
          << " can't read GNUmakefile." 
          << std::endl;
    return false;
  }

  /* remove existing :
  ifdef G4ANALYSIS_USE
   CPPFLAGS += `aida-config --include`
   LDFLAGS  += `aida-config --lib`
   LDLIBS  += `aida-config --lib`
   LOADLIBS += `aida-config --lib`
  endif
  */
  std::vector<std::string> text_1;
 {for(std::vector<std::string>::const_iterator it=text.begin();
      it!=text.end();++it) {
     if(
  ((*it).find(" += `aida-config --include")!=std::string::npos) ||
  ((*it).find(" += `aida-config --lib`")!=std::string::npos) ){
      text_1.push_back("#OSC"+*it);
    } else {
      text_1.push_back(*it);
    }
  }}

  std::vector<std::string> new_text;

 {bool binmake_found = false;
  for(std::vector<std::string>::const_iterator it=text_1.begin();
      it!=text_1.end();++it) {
    if((*it).find("osc-config")!=std::string::npos) {
      //already done.
      a_out << PROGRAM__NAME << " :"
            << " GNUmakefile already treated."
            << std::endl;
      return false;  
    }
    std::string::size_type pos = (*it).find("config/binmake.gmk");
    if(pos!=std::string::npos) {

      new_text.push_back("#OSC:begin");
      new_text.push_back("# OpenScientist compilation and link flags :");
      new_text.push_back("# Disconnect the usage of other GUIs :");
      new_text.push_back("G4UI_NONE := 1");
      new_text.push_back("# Disconnect the usage of G4 vis drivers that may interfer :");
      new_text.push_back("G4VIS_NONE := 1");


      new_text.push_back("ifneq (,$(findstring WIN32-VC,$(G4SYSTEM)))");
      new_text.push_back("  # need a syntax that both g++ and CL.exe understand :");
      new_text.push_back("  G4OSC_TMP_1_1 := $(shell echo \"`osc-config --g4lab_incs`\")");
      new_text.push_back("  CPPFLAGS += $(patsubst /I%,-I\"%\",$(G4OSC_TMP_1_1))");
      new_text.push_back("  G4OSC_TMP_1_2 := $(shell echo \"`osc-config --g4lab_libs`\" | sed -e 's:\\\\:/:g')");
      new_text.push_back("  INTYLIBS += $(patsubst /LIBPATH:%,'-libpath:'%,$(G4OSC_TMP_1_2))");
      new_text.push_back("  G4OSC_TMP_1_3 := $(shell echo \"`osc-config --iv_libs`\" | sed -e 's:\\\\:/:g')");
      new_text.push_back("  INTYLIBS += $(patsubst /LIBPATH:%,'-libpath:'%,$(G4OSC_TMP_1_3))");
      new_text.push_back("else");
      new_text.push_back("  CPPFLAGS += $(shell osc-config --g4lab_incs)");
      new_text.push_back("  INTYLIBS += $(shell osc-config --g4lab_libs)");
      new_text.push_back("  INTYLIBS += $(shell osc-config --iv_libs)");
      new_text.push_back("endif");

      new_text.push_back("");
      new_text.push_back("# Enable OpenScientist AIDA implementation :");
      new_text.push_back("G4ANALYSIS_USE := 1");


      new_text.push_back("ifneq (,$(findstring WIN32-VC,$(G4SYSTEM)))");
      new_text.push_back("  G4OSC_TMP_2 := $(shell echo \"`aida-config --include`\")");
      new_text.push_back("  G4ANALYSIS_AIDA_CONFIG_CFLAGS = $(patsubst /I%,-I\"%\",$(G4OSC_TMP_2))");
      new_text.push_back("else");
      new_text.push_back("  G4ANALYSIS_AIDA_CONFIG_CFLAGS = $(shell aida-config --include)");
      new_text.push_back("endif");

      if(a_aida_vis) {
        new_text.push_back("ifneq (,$(findstring WIN32-VC,$(G4SYSTEM)))");
        new_text.push_back("  G4OSC_TMP_2_1 := $(shell echo \"`OnXLab_aida_config --libs`\" | sed -e 's:\\\\:/:g')");
        new_text.push_back("  G4ANALYSIS_AIDA_CONFIG_LIBS = $(patsubst /LIBPATH:%,'-libpath:'%,$(G4OSC_TMP_2_1))");
        new_text.push_back("else");
        new_text.push_back("  G4ANALYSIS_AIDA_CONFIG_LIBS = $(shell OnXLab_aida_config --libs)");
        new_text.push_back("endif");
      } else {
        new_text.push_back("ifneq (,$(findstring WIN32-VC,$(G4SYSTEM)))");
        new_text.push_back("  G4OSC_TMP_2_1 := $(shell echo \"`BatchLab_aida_config --libs`\" | sed -e 's:\\\\:/:g')");
        new_text.push_back("  G4ANALYSIS_AIDA_CONFIG_LIBS = $(patsubst /LIBPATH:%,'-libpath:'%,$(G4OSC_TMP_2_1))");
        new_text.push_back("else");
        new_text.push_back("  G4ANALYSIS_AIDA_CONFIG_LIBS = $(shell BatchLab_aida_config --libs)");
        new_text.push_back("endif");
      }
      new_text.push_back("#OSC:end");
      new_text.push_back("");

      new_text.push_back(*it);
      binmake_found = true;
    } else {
      new_text.push_back(*it);
    }
  }
  if(!binmake_found) {
    a_out << PROGRAM__NAME << " :"
          << " config/binmake.gmk not found in GNUmakefile." 
          << std::endl;
    return false;
  }}

  if(!Slash::file::copy("GNUmakefile","GNUmakefile_before_osc")) {
    a_out << PROGRAM__NAME << " :"
          << " can't copy GNUmakefile to GNUmakefile_before_osc." 
          << std::endl;
    return false;
  }

  if(!Slash::file::write("GNUmakefile",new_text)) {
    a_out << PROGRAM__NAME << " :"
          << " can't write GNUmakefile." 
          << std::endl;
    return false;
  }

  if(a_verbose) {
    a_out << " GNUmakefile modified." 
          << " Original file copied to GNUmakefile_before_osc."
          << std::endl;
  }

  return true;
}

bool restore_in_dir(const std::string& a_dir,bool a_verbose,std::ostream& a_out) {

  bool status = true;

  // Search "before_osc" files in a_dir directory and copy them back.
  std::string curdir;
  if(!Slash::directory::pwd(curdir)) {
    a_out << PROGRAM__NAME << " :"
          << " can't get current directory."
          << std::endl;
    return false;
  }
  if(!Slash::directory::cd(a_dir)) {
    a_out << PROGRAM__NAME << " :"
          << " can't go in " << Slash::sout(a_dir) << " directory."
          << std::endl;
    return false;
  }

  std::vector<std::string> files;
  if(!Slash::directory::entries(".",files,false)) {
    a_out << PROGRAM__NAME << " :"
          << " can't get file in " << Slash::sout(a_dir) << " directory."
          << std::endl;
    status = false;
    files.clear();
  }

  for(std::vector<std::string>::const_iterator it=files.begin();
      it!=files.end();++it) {
    const std::string& s = *it;
    std::string::size_type pos = s.find("_before_osc");
    if(pos!=std::string::npos) {
      std::string file = s.substr(0,pos);
      if(!Slash::file::copy(s,file)) {
        a_out << PROGRAM__NAME << " :"
              << " can't copy "
              << Slash::sout(s) << " to "
              << Slash::sout(file) << "."
              << std::endl;
        status = false;
        break;
      }
      if(!Slash::file::remove(s)) {
        a_out << PROGRAM__NAME << " :"
              << " can't remove "
              << Slash::sout(s)
              << std::endl;
        status = false;
        break;
      }
      if(a_verbose) {
        a_out << " " << Slash::sout(s)
              << " restored."
              << std::endl;
      }
    }
  }

  if(!Slash::directory::cd(curdir)) {
    a_out << PROGRAM__NAME << " :"
          << " can't go in " << Slash::sout(curdir) << " directory."
          << std::endl;
    return false;
  }

  return status;
}

bool treat_restore(const std::string& a_package,bool a_verbose,std::ostream& a_out) {

  // Search "before_osc" files and copy them back.

  bool status = true;

  if(!restore_in_dir(".",a_verbose,a_out)) {status = false;} //continue
  if(!restore_in_dir("src",a_verbose,a_out)) {status = false;} //continue
  if(!restore_in_dir("include",a_verbose,a_out)) {status = false;}

  if(!Slash::file::remove("main.onx")) {
    a_out << PROGRAM__NAME << " :"
          << " can't remove main.onx."
          << std::endl;
    status = false;  //continue
  }
  if(a_verbose) {
    a_out << " main.onx removed." << std::endl;
  }
  if(!Slash::file::remove("simple_main.onx")) {
    a_out << PROGRAM__NAME << " :"
          << " can't remove simple_main.onx."
          << std::endl;
    status = false;  //continue
  }
  if(a_verbose) {
    a_out << " simple_main.onx removed." << std::endl;
  }
  std::string file_styles = a_package+".styles";
  if(!Slash::file::remove(file_styles)) {
    a_out << PROGRAM__NAME << " :"
          << " can't remove " << Slash::sout(file_styles)
          << std::endl;
    status = false;  //continue
  }
  if(a_verbose) {
    a_out << " " << Slash::sout(file_styles)
          << " removed."
          << std::endl;
  }


  //add_this_obuild

  //add_osc_dll_gmk
  if(!Slash::file::remove("Callbacks.cxx")) {
    a_out << PROGRAM__NAME << " :"
          << " can't remove Callbacks.cxx."
          << std::endl;
    status = false;  //continue
  }
  if(a_verbose) {
    a_out << " Callbacks.cxx removed." << std::endl;
  }
  if(!Slash::file::remove("osc_dll.gmk")) {
    a_out << PROGRAM__NAME << " :"
          << " can't remove osc_dll.gmk."
          << std::endl;
    status = false;  //continue
  }
  if(a_verbose) {
    a_out << " osc_dll.gmk removed." << std::endl;
  }

  return status;
}

bool get_package(std::string& a_package,bool a_verbose,std::ostream& a_out){
  if(!Slash::directory::pwd(a_package)) {
    a_out << PROGRAM__NAME << " :"
          << " can't get directory name."
          << std::endl;
    return false;
  }
  a_package = Slash::nosuffix(a_package);
  if(a_verbose) {
    a_out << " package or example name is " << Slash::sout(a_package)
          << std::endl;
  }
  return true;
}


bool add_this_obuild(const std::string& aFrom,const std::string& a_package,std::ostream& a_out){
  if(!Slash::directory::create("obuild")) {
    a_out << PROGRAM__NAME << " :"
          << " can't create obuild directory."
          << std::endl;
    return false;
  }
  if(!Slash::directory::cd("obuild")) {
    a_out << PROGRAM__NAME << " :"
          << " can't go in obuild directory."
          << std::endl;
    return false;
  }

  std::vector<std::string> changes;

  if(!Slash::file::copy(aFrom+"/this.obuild","this.obuild")) {
    a_out << PROGRAM__NAME << " :"
          << " can't copy " << aFrom << "/this.obuild."
          << std::endl;
    return false;
  }
  if(!file_replace("this.obuild","@@package@@",a_package,changes)) {
    a_out << PROGRAM__NAME << " :"
          << " can't replace in this.obuild."
          << std::endl;
    return false;
  }

  if(!Slash::file::copy(aFrom+"/Callbacks.cxx","Callbacks.cxx")) {
    a_out << PROGRAM__NAME << " :"
          << " can't copy " << aFrom << "/Callbacks.cxx."
          << std::endl;
    return false;
  }
  if(!file_replace("Callbacks.cxx","@@package@@",a_package,changes)) {
    a_out << PROGRAM__NAME << " :"
          << " can't replace in Callbacks.cxx."
          << std::endl;
    return false;
  }

  if(!Slash::directory::cd("..")) {
    a_out << PROGRAM__NAME << " :"
          << " can't go in obuild directory."
          << std::endl;
    return false;
  }

  return true;
}

bool add_osc_dll_gmk(const std::string& aFrom,const std::string& a_package,std::ostream& a_out){
  std::vector<std::string> changes;

  if(!Slash::file::copy(aFrom+"/osc_dll.gmk","osc_dll.gmk")) {
    a_out << PROGRAM__NAME << " :"
          << " can't copy " << aFrom << "/osc_dll.gmk."
          << std::endl;
    return false;
  }
  if(!file_replace("osc_dll.gmk","@@package@@",a_package,changes)) {
    a_out << PROGRAM__NAME << " :"
          << " can't replace in osc_dll.gmk."
          << std::endl;
    return false;
  }

  if(!Slash::file::copy(aFrom+"/Callbacks.cxx","Callbacks.cxx")) {
    a_out << PROGRAM__NAME << " :"
          << " can't copy " << aFrom << "/Callbacks.cxx."
          << std::endl;
    return false;
  }
  if(!file_replace("Callbacks.cxx","@@package@@",a_package,changes)) {
    a_out << PROGRAM__NAME << " :"
          << " can't replace in Callbacks.cxx."
          << std::endl;
    return false;
  }

  return true;
}

#include <iostream>

int main(int aArgc,char** aArgv) {

  Slash::args args(aArgc,aArgv);

 {std::vector<std::string> options;
  options.push_back("-h");
  options.push_back("-aida_batch");
  options.push_back("-verbose");
  options.push_back("-restore");
  if(!args.known_options(options)) {
    std::cout << PROGRAM__NAME << " :"
        << " Some options are unknown." << std::endl;
    std::cout << "Passed options was :" << std::endl;
    std::cout << Slash::to_string(args.tovector()," ") << std::endl;
    std::cout << "To have help, type :" << std::endl;
    std::cout << "  OS> " << PROGRAM__NAME << " -h" << std::endl;
    return EXIT_FAILURE;
  }}

  if(args.is_arg("-h")) {
    std::cout << "\
\n\
 This program attempts to instrument automatically\n\
some of the Geant4 examples (like the novice ones)\n\
in order to use G4Lab::UIOnX to build an OnX GUI\n\
for them.\n\
 It should work also on user code which is a simple\n\
cloning of one of the Geant4 example.\n\
\n\
 This program changes the file containing the main\n\
in order to instanciate the G4Lab::UIOnX instead\n\
of the basic G4UIterminal.\n\
 It changes also the GNUmakefile in order to have\n\
the correct compilation and link flags.\n\
 A copy of the original files is done with the\n\
suffix \"_before_osc\". Note that modifications\n\
are tagged with some [//,#]OSC:[begin,end] comments\n\
in order to easily trace them.\n\
\n\
 Since OpenScientist comes also with an AIDA\n\
implementation, the programs put also in the\n\
GNUmakefile the necessary material to use this\n\
AIDA implementation. By default the \"vis\" AIDA\n\
implementation of OpenScientist is used in order\n\
to have also the plotting within the same interactive\n\
application.\n\
 In case you do not want to use the AIDA plotting,\n\
use the -aida_batch option. This option induces\n\
no modifications in the \"analysis manager\" class in order to\n\
pass the AIDA::IAnalysisFactory of the G4Lab::UIOnX.\n\
In this case it is the AIDA \"batch\" implementation\n\
of OpenScientist which is used.\n\
" << std::endl;
    std::cout << "Usage :" << std::endl;
    std::cout << "  OS> " << PROGRAM__NAME << std::endl;
    std::cout << "  OS> " << PROGRAM__NAME << " -aida_batch" << std::endl;
    std::cout << "  OS> " << PROGRAM__NAME << " -h" << std::endl;
    std::cout << "  OS> " << PROGRAM__NAME << " -verbose" << std::endl;
    std::cout << "  OS> " << PROGRAM__NAME << " -restore" << std::endl;
    return EXIT_SUCCESS;
  }

  std::string from;
 {std::string OSC_HOME_DIR;
  if(!Slash::getenv("OSC_HOME_DIR",OSC_HOME_DIR)) {
    std::cout << PROGRAM__NAME << " :"
        << " cant' guess where OpenScientist had been installed."
        << " (OSC_HOME_DIR environment variable is not set)."
        << std::endl;
  } else {
    from = OSC_HOME_DIR+"/Resources/G4Lab/applications/G4Lab_g4_to_osc";
  }}

  bool verbose = args.is_arg("-verbose");
  bool restore = args.is_arg("-restore");

  //package/example name :
  std::string package;
  if(!get_package(package,verbose,std::cout)) {
    return EXIT_FAILURE;
  }

  if(restore) {
    if(!treat_restore(package,verbose,std::cout)) {
      return EXIT_FAILURE;
    } else {
      if(verbose) {
        std::cout << " Restore is a success."
            << std::endl;
      }
      return EXIT_SUCCESS;
    }
  }

  bool aida_vis = args.is_arg("-aida_batch")?false:true;

  std::string am_hh;
  std::string am_cc;
  std::string am_class;
  if(aida_vis) {
    //search source file containing a AIDA_createAnalysisFactory()
    if(!find_am_cc(am_cc,std::cout)) {
      return EXIT_FAILURE;
    }
    if(verbose) {
      if(am_cc.size()) {
        std::cout << " Found file " << Slash::sout(am_cc)
            << " containing AIDA_createAnalysisFactory()."
            << std::endl;
      } else {
        std::cout << " No file found containing AIDA_createAnalysisFactory()."
            << std::endl;
      }
    }
    if(am_cc.size()) {
      am_class = Slash::nosuffix(am_cc);
      if(verbose) {
        std::cout << " Assume that the analysis manager class is " << am_class
            << std::endl;
      }   
      if(!find_am_hh(am_class,am_hh,std::cout)) {
        std::cout << "Unable to find header file for class " << am_class
            << std::endl;
        return EXIT_FAILURE;        
      }
    }   
  }

  if(am_hh.empty()) aida_vis = false; //no analysis manager found.
 
  // example main file :
  std::string exa_name;
 {if(!find_name_in_GNUmakefile(exa_name,verbose,std::cout)) 
    return EXIT_FAILURE;
  std::string main_file = exa_name+".cc";
  if(!Slash::file::exists(main_file)) {
    std::cout << PROGRAM__NAME << " :"
        << " file " << Slash::sout(main_file) << " not found."
        << std::endl;
    return EXIT_FAILURE;
  }
  if(verbose) {
     std::cout << " main file is " << Slash::sout(main_file) << "."
         << std::endl;
  }
  bool interactif;
  if(!find_SessionStart(main_file,interactif,std::cout))
    return EXIT_FAILURE;
  if(!interactif) {
    std::cout << " No SessionStart() found in main file."
        << " We assume a pure batch program and then do nothing."
        << std::endl;
    return EXIT_SUCCESS;
  }
  if(!treat_main(package,
                 main_file,aida_vis,
                 am_hh,am_class,
                 verbose,std::cout)) return EXIT_FAILURE;}

  if(am_cc.size() && am_hh.size() && am_class.size()) {
    if(!treat_am_cc(am_cc,am_class,verbose,std::cout)) 
      return EXIT_FAILURE;
    if(!treat_am_hh(am_hh,am_class,verbose,std::cout)) 
      return EXIT_FAILURE;
  }

  if(!treat_GNUmakefile(aida_vis,verbose,std::cout)) 
    return EXIT_FAILURE;

 {if(!Slash::file::copy(from+"/simple_main.onx","simple_main.onx")) {
    std::cout << PROGRAM__NAME << " :"
        << " can't copy " << from << "/simple_main.onx."
        << std::endl;
    return EXIT_FAILURE;
  }
  std::vector<std::string> changes;
  if(!file_replace("simple_main.onx","@@package@@",package,changes)) {
    std::cout << PROGRAM__NAME << " :"
        << " can't replace in simple_main.onx."
        << std::endl;
    return false;
  }}
 {if(!Slash::file::copy(from+"/main.onx","main.onx")) {
    std::cout << PROGRAM__NAME << " :"
        << " can't copy " << from << "/main.onx."
        << std::endl;
    return EXIT_FAILURE;
  }
  std::vector<std::string> changes;
  if(!file_replace("main.onx","@@package@@",package,changes)) {
    std::cout << PROGRAM__NAME << " :"
        << " can't replace in main.onx."
        << std::endl;
    return false;
  }}
  if(!Slash::file::copy(from+"/package.styles",package+".styles")) {
    std::cout << PROGRAM__NAME << " :"
        << " can't copy " << from << "/package.styles."
        << std::endl;
    return EXIT_FAILURE;
  }

  if(!Slash::file::copy(from+"/app.help","app.help")) {
    std::cout << PROGRAM__NAME << " :"
        << " can't copy " << from << "/app.help."
         << std::endl;
    return EXIT_FAILURE;
  }

  if(package=="TestEm1") {
    if(!Slash::file::copy(from+"/TestEm1_histos.mac","osc_default.mac")) {
      std::cout << PROGRAM__NAME << " :"
          << " can't copy " << from << "/TestEm1_histos.mac."
         << std::endl;
      return EXIT_FAILURE;
    }

    std::string ss;
    for(unsigned int index=0;index<session_start_cxx_linen;index++) {
      std::string s = session_start_cxx_lines[index];
      Slash::replace(s,"@@double_quote@@","\"");
      Slash::replace(s,"@@back_slash@@","\\");
      ss += "\n";
      ss += s;
    }
    std::vector<std::string> changes;
    if(!file_replace("TestEm1.cc","session->SessionStart();",ss,changes)) {
      std::cout << PROGRAM__NAME << " :"
          << " can't replace in TestEm1.cc."
          << std::endl;
      return false;
    }
  }

 {if(!Slash::file::copy(from+"/build_Darwin_app","build_Darwin_app")) {
    std::cout << PROGRAM__NAME << " :"
        << " can't copy " << from << "/build_Darwin_app."
        << std::endl;
    return EXIT_FAILURE;
  }
  std::vector<std::string> changes;
  if(!file_replace("build_Darwin_app","@@name@@",exa_name,changes)) {
    std::cout << PROGRAM__NAME << " :"
        << " can't replace in main.onx."
        << std::endl;
    return false;
  }
  Slash::file::chmod("build_Darwin_app",
                     Slash::file::USER,
                     Slash::file::ADD,
		     Slash::file::EXECUTE);}

  //if(!add_this_obuild(from,package,std::cout)) 
  //  return EXIT_FAILURE;
  if(!add_osc_dll_gmk(from,package,std::cout)) 
    return EXIT_FAILURE;

  return EXIT_SUCCESS;
}
