@ECHO OFF
SETLOCAL

REM # Check that onx-config is available :
onx-config 1> NUL 2> NUL
IF ERRORLEVEL 1 (
  ECHO onx-config program not found.
  GOTO build_return
)

REM # Check that aida-config is available :
aida-config 1> NUL 2> NUL
IF ERRORLEVEL 1 (
  ECHO aida-config program not found.
  GOTO build_return
)

IF NOT DEFINED CLHEP_BASE_DIR (
  ECHO Environment variable CLHEP_BASE_DIR not defined.
  GOTO build_return
)

IF NOT DEFINED G4INSTALL (
  ECHO Environment variable G4INSTALL not defined.
  GOTO build_return
)

SET bin=.

REM # /GR to enable RTTI and dynamic cast.
REM # /GX to enable exception handling (for STL).
REM # /O2 optimize
REM # /W3 warmings
SET def_cppflags=/nologo /DWIN32 /MD /GX /GR /O2 /W3

FOR /F "usebackq delims=;" %%s IN (`onx-config --cppflags`) DO SET onx_cppflags=%%s
FOR /F "usebackq delims=;" %%s IN (`onx-config --libs`) DO SET onx_libs=%%s
FOR /F "usebackq delims=;" %%s IN (`onx-config --iv_cppflags`) DO SET onx_iv_cppflags=%%s
FOR /F "usebackq delims=;" %%s IN (`onx-config --iv_libs`) DO SET onx_iv_libs=%%s
FOR /F "usebackq delims=;" %%s IN (`onx-config --core_libs`) DO SET onx_core_libs=%%s
FOR /F "usebackq delims=;" %%s IN (`aida-config --cppflags`) DO SET aida_cppflags=%%s
FOR /F "usebackq delims=;" %%s IN (`aida-config --libs`) DO SET aida_libs=%%s
FOR /F "usebackq delims=;" %%s IN (`onx-config --g4lab_cppflags`) DO SET onx_g4lab_cppflags=%%s
FOR /F "usebackq delims=;" %%s IN (`onx-config --g4lab_libs`) DO SET onx_g4lab_libs=%%s
FOR /F "usebackq delims=;" %%s IN (`onx-config --g4_cppflags`) DO SET onx_g4_cppflags=%%s
FOR /F "usebackq delims=;" %%s IN (`onx-config --g4_libs`) DO SET onx_g4_libs=%%s

REM #////////////////////////////////////////////////////////////////
REM #// substitute %CLHEP_BASE_DIR%, %G4INSTALL% by their value :
REM #////////////////////////////////////////////////////////////////
FOR /F "usebackq delims=;" %%s IN (`ECHO %onx_g4lab_cppflags%`) DO SET onx_g4lab_cppflags=%%s
FOR /F "usebackq delims=;" %%s IN (`ECHO %onx_g4lab_libs%`) DO SET onx_g4lab_libs=%%s
FOR /F "usebackq delims=;" %%s IN (`ECHO %onx_g4_cppflags%`) DO SET onx_g4_cppflags=%%s
FOR /F "usebackq delims=;" %%s IN (`ECHO %onx_g4_libs%`) DO SET onx_g4_libs=%%s

REM #//////////////////////////////////////////////////
REM #// library :
REM #//////////////////////////////////////////////////
SET cppflags=%def_cppflags%
SET cppflags=%cppflags% /I..\include
SET cppflags=%cppflags% %onx_g4lab_cppflags%
SET cppflags=%cppflags% %aida_cppflags%
cl.exe /c %cppflags% /Fo%bin%\Detector.obj /Tp..\src\Detector.cxx
cl.exe /c %cppflags% /Fo%bin%\PhysicsList.obj /Tp..\src\PhysicsList.cxx
cl.exe /c %cppflags% /Fo%bin%\Primary.obj /Tp..\src\Primary.cxx
cl.exe /c %cppflags% /Fo%bin%\EventAction.obj /Tp..\src\EventAction.cxx
cl.exe /c %cppflags% /Fo%bin%\RunAction.obj /Tp..\src\RunAction.cxx
cl.exe /c %cppflags% /Fo%bin%\SteppingAction.obj /Tp..\src\SteppingAction.cxx
cl.exe /c %cppflags% /Fo%bin%\Analysis.obj /Tp..\src\Analysis.cxx

SET objs=%bin%\Detector.obj %bin%\PhysicsList.obj %bin%\Primary.obj %bin%\EventAction.obj %bin%\RunAction.obj %bin%\SteppingAction.obj %bin%\Analysis.obj

IF EXIST %bin%\libChunk.arc DEL %bin%\libChunk.arc
lib.exe /nologo /machine:ix86 /out:%bin%\libChunk.arc %objs%

DEL %objs%

REM #//////////////////////////////////////////////////
REM #// vis :
REM #//////////////////////////////////////////////////
SET cppflags=%def_cppflags%
SET cppflags=%cppflags% /I..\include
SET cppflags=%cppflags% %onx_cppflags%
SET cppflags=%cppflags% %onx_iv_cppflags%
SET cppflags=%cppflags% %aida_cppflags%
SET cppflags=%cppflags% %onx_g4lab_cppflags%
cl.exe /c %cppflags% /Fo%bin%\vis.obj /Tp..\apps\vis.cxx

SET libs=%onx_iv_libs%
SET libs=%libs% %onx_core_libs%
SET libs=%libs% %onx_libs%
SET libs=%libs% %onx_g4lab_libs%
SET libs=%libs% libG4physicslists.a

IF EXIST %bin%\vis.exe DEL %bin%\vis.exe
link.exe /nologo /out:%bin%\vis.exe %bin%\vis.obj %bin%\libChunk.arc %libs%

REM #//////////////////////////////////////////////////
REM #// batch :
REM #//////////////////////////////////////////////////
SET cppflags=%def_cppflags%
SET cppflags=%cppflags% /I..\include
SET cppflags=%cppflags% %onx_cppflags%
SET cppflags=%cppflags% %aida_cppflags%
SET cppflags=%cppflags% %onx_g4_cppflags%
cl.exe /c %cppflags% /Fo%bin%\batch.obj /Tp..\apps\batch.cxx

SET libs=%onx_iv_libs%
REM SET libs=%libs% %onx_core_libs%
SET libs=%libs% %onx_libs%
SET libs=%libs% %onx_g4_libs%
SET libs=%libs% %aida_libs%
SET libs=%libs% libG4physicslists.a

IF EXIST %bin%\batch.exe DEL %bin%\batch.exe
link.exe /nologo /out:%bin%\batch.exe %bin%\batch.obj %bin%\libChunk.arc %libs%

REM #//////////////////////////////////////////////////
REM #// analysis :
REM #//////////////////////////////////////////////////
SET cppflags=%def_cppflags%
SET cppflags=%cppflags% %aida_cppflags%
cl.exe /c %cppflags% /Fo%bin%\analysis.obj /Tp..\apps\analysis.cxx

IF EXIST %bin%\analysis.exe DEL %bin%\analysis.exe
link.exe /nologo /out:%bin%\analysis.exe %bin%\analysis.obj %libs% %aida_libs%

REM #//////////////////////////////////////////////////
REM #// end :
REM #//////////////////////////////////////////////////

DEL /Q *.obj

:build_return

ENDLOCAL
@ECHO ON
