
// this :
#include <Detector.h>

// Geant4 :
#include <G4Element.hh>
#include <G4Material.hh>
#include <G4LogicalVolume.hh>
#include <G4PVPlacement.hh>

#include <G4Box.hh>
#include <G4Tubs.hh>

Chunk::Detector::Detector(G4Material& aMaterial)
:fMaterial(aMaterial)
,f_chunk_PV(0)
{}                
Chunk::Detector::~Detector() {}

G4VPhysicalVolume* Chunk::Detector::Construct() {        

  G4Element* N_el = new G4Element("Nitrogen","N",7,14.01*g/mole);
  G4Element* O_el = new G4Element("Oxygen","O",8,16.00*g/mole);

  G4Material* air_mat = new G4Material("Air",1.29e-03*g/cm3,2);
  air_mat->AddElement(N_el,.7);
  air_mat->AddElement(O_el,.3);

  G4LogicalVolume* world_LV = 
    new G4LogicalVolume(new G4Box("",10000*cm,10000*cm,10000*cm),
                        air_mat,"world_LV",0,0,0);
  G4VPhysicalVolume* world_PV = 
    new G4PVPlacement(0,G4ThreeVector(),"world_PV",world_LV,0,false,0);

  double radlen = fMaterial.GetRadlen();

  //G4double w_2 = 40*radlen;
  //G4double h_2 = 40*radlen;
    //new G4LogicalVolume(new G4Box("",w_2,h_2,d_2), //Half sizes.

  G4double R = 40*radlen;
  G4double d_2 = 20*radlen;
  G4LogicalVolume* chunk_LV = 
    new G4LogicalVolume(new G4Tubs("",0,R,d_2,0*deg,360*deg),
                        &fMaterial,"chunk_LV",0,0,0);
  f_chunk_PV = 
    new G4PVPlacement(0,G4ThreeVector(0,0,d_2),
                      "chunk_PV",chunk_LV,world_PV,false,1);
  
  return world_PV;
}
G4VPhysicalVolume* Chunk::Detector::chunck_PV() const {        
  return f_chunk_PV;
}
double Chunk::Detector::radiation_length() const {        
  return fMaterial.GetRadlen();
}
