// This :
#include <G4LabTemplateOne/CellHit.h>

// Lib :
#include <Lib/Debug.h>
#include <Lib/smanip.h>
 
// Geant4 :
#include <G4Allocator.hh>
#include <G4UnitsTable.hh>
#include <G4ios.hh>
#include <G4AttDefStore.hh>
#include <G4AttDef.hh>
#include <G4AttValue.hh>

static G4Allocator<G4LabTemplateOne::CellHit> sHitAllocator; //Beurk.

void* G4LabTemplateOne::CellHit::operator new(size_t) {
  return (void*)sHitAllocator.MallocSingle();
}

void G4LabTemplateOne::CellHit::operator delete(void* aHit)
{
  sHitAllocator.FreeSingle((G4LabTemplateOne::CellHit*)aHit);
}

G4LabTemplateOne::CellHit::CellHit():fLV(0),fEnergy(0){
  Lib::Debug::increment("G4LabTemplateOne::CellHit");
}

G4LabTemplateOne::CellHit::CellHit(const G4LabTemplateOne::CellHit& aFrom):G4VHit() {
  fEnergy = aFrom.fEnergy;
  fLV = aFrom.fLV;
  fTSF = aFrom.fTSF;
  Lib::Debug::increment("G4LabTemplateOne::CellHit");
}

G4LabTemplateOne::CellHit::~CellHit() {
  Lib::Debug::decrement("G4LabTemplateOne::CellHit");
}

const G4LabTemplateOne::CellHit& G4LabTemplateOne::CellHit::operator=(
  const G4LabTemplateOne::CellHit& aFrom) {
  fEnergy = aFrom.fEnergy;
  fLV = aFrom.fLV;
  fTSF = aFrom.fTSF;
  return *this;
}

G4int G4LabTemplateOne::CellHit::operator==(const G4LabTemplateOne::CellHit& aFrom) const {
  return (this==&aFrom) ? 1 : 0;
}

void G4LabTemplateOne::CellHit::Draw() {}

void G4LabTemplateOne::CellHit::Print() {
  G4cout << " Energy " << G4BestUnit(fEnergy,"Energy") << G4endl;
}

const std::map<G4String,G4AttDef>* G4LabTemplateOne::CellHit::GetAttDefs() const {
  G4bool isNew;
  std::map<G4String,G4AttDef>* attDefs = 
    G4AttDefStore::GetInstance("CellHit",isNew);

  if (isNew) {    
    G4String Energy("Energy");
    (*attDefs)[Energy] = G4AttDef(Energy,"Energy","Dump","","G4double");    

    G4String LV("LV");
    (*attDefs)[LV] = 
      G4AttDef(LV,"Logical Volume","Display","","G4LogicalVolume");

    G4String TSF("TSF");
    (*attDefs)[TSF] = 
      G4AttDef(TSF,"3D Transformation","Display","","G4Transform3D");
 
  }

  return attDefs;
}

std::vector<G4AttValue>* G4LabTemplateOne::CellHit::CreateAttValues() const {

  std::vector<G4AttValue>* values = new std::vector<G4AttValue>;

  std::ostringstream os;
  os.str("");
  os << fEnergy;
  values->push_back(G4AttValue("Energy",os.str(),""));
    
  values->push_back(G4AttValue("LV",Lib::smanip::p2s((void*)fLV),""));

  values->push_back(G4AttValue("TSF",Lib::smanip::p2s((void*)&fTSF),""));

  return values;
}

void G4LabTemplateOne::CellHit::SetLV(G4LogicalVolume* aLV) { 
  fLV = aLV; 
}
void G4LabTemplateOne::CellHit::SetTSF(const G4Transform3D& aTSF) { 
  fTSF = aTSF; 
}
void G4LabTemplateOne::CellHit::SetEnergy(G4double aEnergy) {
  fEnergy = aEnergy;
}
G4double G4LabTemplateOne::CellHit::GetEnergy() const {
  return fEnergy;
}
