#ifndef HEPVis_SbColormap_h
#define HEPVis_SbColormap_h

#include <vector>
#include <string>

#include <Inventor/SbColor.h>

#include <HEPVis/SbStyle.h> //FIXME

class SbBaseColormap {
public:
  virtual SbColor getColor(float a_ratio) const = 0;
  virtual void* cast(const std::string&) const = 0;
public:
  SbBaseColormap();
  SbBaseColormap(const SbBaseColormap& aFrom);
  SbBaseColormap& operator=(const SbBaseColormap&);
  virtual ~SbBaseColormap();
public:
  unsigned int colorn() const;
  unsigned int valn() const;
  SbColor color(unsigned int) const;
  float value(unsigned int) const;
public:
  // helper :
  void setColors(SbColor(*aGet)(float),unsigned int aCelln);
  void set_PAW_coloring();
protected:
  std::vector<float> fValues;
  std::vector<SbColor> fColors;
};

#define if_Sb_SCast(a__class) if(a_class==#a__class) {return (void*)static_cast<const a__class*>(this);}

class SbByValueColormap : public SbBaseColormap {
public:
  virtual SbColor getColor(float a_value) const{
    return SbStyle::getByValue(a_value,fValues,fColors);
  }
  virtual void* cast(const std::string& a_class) const {
    if_Sb_SCast(SbByValueColormap)
    else if_Sb_SCast(SbBaseColormap)
    else return 0;
  }
public:
  SbByValueColormap(const SbString& aString){
    SbStyle::setByValue(aString,fValues,fColors);
  }
  SbByValueColormap(const SbByValueColormap& aFrom):SbBaseColormap(aFrom){}
  SbByValueColormap& operator=(const SbByValueColormap& aFrom){ 
    SbBaseColormap::operator=(aFrom);
    return *this;
  }
  virtual ~SbByValueColormap(){}
};

class SbGreyScaleColormap : public SbBaseColormap {
public:
  virtual SbColor getColor(float a_value) const{ //a_value in [0,1]
    return SbStyle::getGrey(a_value);
  }
  virtual void* cast(const std::string& a_class) const {
    if_Sb_SCast(SbGreyScaleColormap)
    else if_Sb_SCast(SbBaseColormap)
    else return 0;
  }
public:
  SbGreyScaleColormap(float a_min,float a_max,unsigned int aCelln){
    fValues.resize(2);
    fValues[0] = a_min;
    fValues[1] = a_max;
    setColors(SbStyle::getGrey,aCelln);
  }
  SbGreyScaleColormap(const SbGreyScaleColormap& aFrom):SbBaseColormap(aFrom){}
  SbGreyScaleColormap& operator=(const SbGreyScaleColormap& aFrom){ 
    SbBaseColormap::operator=(aFrom);
    return *this;
  }
  virtual ~SbGreyScaleColormap(){}
};

class SbGreyScaleInverseColormap : public SbBaseColormap {
public:
  virtual SbColor getColor(float a_value) const{ //a_value in [0,1]
    return SbStyle::getGreyInverse(a_value);
  }
  virtual void* cast(const std::string& a_class) const {
    if_Sb_SCast(SbGreyScaleInverseColormap)
    else if_Sb_SCast(SbBaseColormap)
    else return 0;
  }
public:
  SbGreyScaleInverseColormap(float a_min,float a_max,unsigned int aCelln){
    fValues.resize(2);
    fValues[0] = a_min;
    fValues[1] = a_max;
    setColors(SbStyle::getGreyInverse,aCelln);
  }
  SbGreyScaleInverseColormap(const SbGreyScaleInverseColormap& aFrom):SbBaseColormap(aFrom){}
  SbGreyScaleInverseColormap& operator=(const SbGreyScaleInverseColormap& aFrom){ 
    SbBaseColormap::operator=(aFrom);
    return *this;
  }
  virtual ~SbGreyScaleInverseColormap(){}
};

class SbVioletToRedColormap : public SbBaseColormap {
public:
  virtual SbColor getColor(float a_value) const { //a_value in [0,1]
    return SbStyle::getVioletToRed(a_value);
  }
  virtual void* cast(const std::string& a_class) const {
    if_Sb_SCast(SbVioletToRedColormap)
    else if_Sb_SCast(SbBaseColormap)
    else return 0;
  }
public:
  SbVioletToRedColormap(float a_min,float a_max,unsigned int aCelln){
    fValues.resize(2);
    fValues[0] = a_min;
    fValues[1] = a_max;
    setColors(SbStyle::getVioletToRed,aCelln);
  }
  SbVioletToRedColormap(const SbVioletToRedColormap& aFrom):SbBaseColormap(aFrom){}
  SbVioletToRedColormap& operator=(const SbVioletToRedColormap& aFrom){ 
    SbBaseColormap::operator=(aFrom);
    return *this;
  }
  virtual ~SbVioletToRedColormap(){}
};

class SbConstColormap : public SbBaseColormap {
public:
  virtual SbColor getColor(float) const {return fColors[0];}
  virtual void* cast(const std::string& a_class) const {
    if_Sb_SCast(SbConstColormap)
    else if_Sb_SCast(SbBaseColormap)
    else return 0;
  }
public:
  SbConstColormap(const SbColor& aColor){fColors.push_back(aColor);}
  SbConstColormap(const SbConstColormap& aFrom):SbBaseColormap(aFrom){}
  SbConstColormap& operator=(const SbConstColormap& aFrom){ 
    SbBaseColormap::operator=(aFrom);
    return *this;
  }
  virtual ~SbConstColormap(){}
};

#undef if_Sb_SCast

#endif
