#ifndef Lib_AccessorManager_h
#define Lib_AccessorManager_h

#include <Slash/Core/IManager.h>
#include <Slash/Data/IProcessor.h>
#include <Slash/Core/IAction.h>

#include <Lib/Handler.h>

#include <list>

namespace Lib {

class AccessorManager 
:public virtual Slash::Core::IManager
,public virtual Slash::Data::IProcessor {
public: //Slash::Core::IManager
  virtual std::string name() const;
  virtual void* cast(const std::string&) const;
public: //IProcessor
  virtual Slash::Core::IWriter& printer();
  virtual void setVerboseLevel(int);
  virtual int verboseLevel() const;

  virtual void addAccessor(Slash::Data::IAccessor*);
  virtual void removeAccessor(const std::string&);
  virtual Slash::Data::IAccessor* findAccessor(const std::string&);
  virtual std::vector<std::string> accessorNames() const;
  virtual std::vector<Slash::Data::IAccessor*> accessors() const;

  virtual void addAction(Slash::Core::IAction*);
  virtual void removeAction(const std::string&);
  virtual std::vector<std::string> actionNames() const;
  virtual std::string execute(const std::string&,
                              const std::vector<std::string>&);

  virtual void addHandler(Slash::Data::IAccessor*,Slash::Data::IAccessor::Data,void*);
  virtual unsigned int numberOfHandlers() const;
  virtual Slash::Data::IHandlersIterator* handlersIterator() const;
  virtual void eraseHandlers();
  virtual void addOutHandler(Slash::Data::IAccessor*,Slash::Data::IAccessor::Data,void*);
  virtual Slash::Data::IHandlersIterator* outHandlersIterator() const;
  virtual void eraseOutHandlers();
  virtual void swapHandlers();
  virtual std::vector<Slash::Data::IAccessor*> handlersAccessors() const;
  virtual bool type_id_tag(const std::string&,
                           Slash::Data::IAccessor*&,void*&,void*&);
public:
  AccessorManager(Slash::Core::IWriter&);
  virtual ~AccessorManager();
public:
  void removeAccessor(Slash::Data::IAccessor*);
  // Actions :
  Slash::Core::IAction* findAction(const std::string&);
protected:
  std::list<Handler> fHandlers;
  std::list<Handler> fOutHandlers;
private:
  Slash::Core::IWriter& fPrinter;
  std::string fName;
  std::list<Slash::Data::IAccessor*> fAccessors;
  std::list<Slash::Core::IAction*> fActions;
  int fVerboseLevel;
};

class CollectAction : public virtual Slash::Core::IAction {
public: //IAction
  virtual void* cast(const std::string&) const;
  virtual std::string name() const;
  virtual std::string execute(const std::vector<std::string>&);
public:
  CollectAction(Slash::Data::IProcessor&);
  virtual ~CollectAction();
private:
  Slash::Data::IProcessor& fAccessorManager;
  std::string fName;
};

class FilterAction : public virtual Slash::Core::IAction {
public: //IAction
  virtual void* cast(const std::string&) const;
  virtual std::string name() const;
  virtual std::string execute(const std::vector<std::string>&);
public:
  FilterAction(Slash::Data::IProcessor&);
  virtual ~FilterAction();
private:
  Slash::Data::IProcessor& fAccessorManager;
  std::string fName;
};

class DumpAction : public virtual Slash::Core::IAction {
public: //IAction
  virtual void* cast(const std::string&) const;
  virtual std::string name() const;
  virtual std::string execute(const std::vector<std::string>&);
public:
  DumpAction(Slash::Data::IProcessor&);
  virtual ~DumpAction();
private:
  Slash::Data::IProcessor& fAccessorManager;
  std::string fName;
};

class NumberAction : public virtual Slash::Core::IAction {
public: //IAction
  virtual void* cast(const std::string&) const;
  virtual std::string name() const;
  virtual std::string execute(const std::vector<std::string>&);
public:
  NumberAction(Slash::Data::IProcessor&);
  virtual ~NumberAction();
private:
  Slash::Data::IProcessor& fAccessorManager;
  std::string fName;
};

class GetValuesAction : public virtual Slash::Core::IAction {
public: //IAction
  virtual void* cast(const std::string&) const;
  virtual std::string name() const;
  virtual std::string execute(const std::vector<std::string>&);
public:
  GetValuesAction(Slash::Data::IProcessor&);
  virtual ~GetValuesAction();
private:
  Slash::Data::IProcessor& fAccessorManager;
  std::string fName;
};

class DestroyAction : public virtual Slash::Core::IAction {
public: //IAction
  virtual void* cast(const std::string&) const;
  virtual std::string name() const;
  virtual std::string execute(const std::vector<std::string>&);
public:
  DestroyAction(Slash::Data::IProcessor&);
  virtual ~DestroyAction();
private:
  Slash::Data::IProcessor& fAccessorManager;
  std::string fName;
};

class SetAction : public virtual Slash::Core::IAction {
public: //IAction
  virtual void* cast(const std::string&) const;
  virtual std::string name() const;
  virtual std::string execute(const std::vector<std::string>&);
public:
  SetAction(Slash::Data::IProcessor&);
  virtual ~SetAction();
private:
  Slash::Data::IProcessor& fAccessorManager;
  std::string fName;
};

class CalculateAction : public virtual Slash::Core::IAction {
public: //IAction
  virtual void* cast(const std::string&) const;
  virtual std::string name() const;
  virtual std::string execute(const std::vector<std::string>&);
public:
  CalculateAction(Slash::Data::IProcessor&);
  virtual ~CalculateAction();
private:
  Slash::Data::IProcessor& fAccessorManager;
  std::string fName;
};

class VisualizeAction : public virtual Slash::Core::IAction {
public: //Slash::Core::IAction
  virtual void* cast(const std::string&) const;
  virtual std::string name() const;
  virtual std::string execute(const std::vector<std::string>&);
public:
  VisualizeAction(Slash::Data::IProcessor&);
  virtual ~VisualizeAction();
private:
  Slash::Data::IProcessor& fAccessorManager;
  std::string fName;
};

class InitAction : public virtual Slash::Core::IAction {
public: //IAction
  virtual void* cast(const std::string&) const;
  virtual std::string name() const;
  virtual std::string execute(const std::vector<std::string>&);
public:
  InitAction(Slash::Data::IProcessor&);
  virtual ~InitAction();
private:
  Slash::Data::IProcessor& fAccessorManager;
  std::string fName;
};

}

#endif



