#ifndef Lib_cstring_h
#define Lib_cstring_h 

#include <string.h> /* strcpy */
#include <stdlib.h> /* malloc,free */

namespace Lib {
inline char* strdup(const char* a_cstr) {
  if(a_cstr==NULL) return NULL; 
  return ::strcpy((char*)::malloc(::strlen(a_cstr)+1),a_cstr);
}
inline void strdel(char*& a_cstr) {
  if(a_cstr==NULL) return;
  ::free(a_cstr);
  a_cstr = NULL;
}
inline bool strempty(const char* a_cstr) {
  return ((a_cstr&&(*a_cstr))?false:true);
}
inline bool streq(const char* a_1,const char* a_2) {
  return ::strcmp(a_1,a_2)?false:true;
}
}

#include <vector>

namespace Lib {

inline unsigned int strtods(const char* a_cstr,std::vector<double>& a_vals,bool a_fixed = false){
  // a_fixed = false : a_vals size is adapted according to what is read.
  // a_fixed = true : a_vals size is fixed. If less is read, then the
  //                 rest is pad with 0.
  // return : the number of double read.

  unsigned int n = a_vals.size();
  char* begin = (char*)a_cstr;
  char* end;
  double d;
  unsigned int index = 0;
  std::vector<double>::iterator it = a_vals.begin();
  while(true) {  
    d = ::strtod(begin,&end);
    if(end==begin) {
      if(a_fixed) {
        for(;it!=a_vals.end();++it) *it = 0;
      } else {
        if(index<n) a_vals.resize(index);
      }
      return index;
    }
    if(index<n) {
      *it = d;
      it++;
    } else {
      if(a_fixed) return index;
      a_vals.push_back(d);
    }
    index++;
    begin = end;
  }
}
}

#include <string>
// Can save a lot of time compared to if(aFrom==aTo) if
// aTo is of the form "xxx". (Because the below avoids
// the copy of aTo in an temporary std::string.
namespace Lib {
inline bool eq(const std::string& a_std_str,const char* a_cstr) {
  return (::strcmp(a_std_str.c_str(),a_cstr)==0?true:false);
}
}

//backward compatibility :
inline char* Lib_strdup(const char* a_cstr) {return Lib::strdup(a_cstr);}
inline void Lib_strdel(char*& a_cstr) {Lib::strdel(a_cstr);}
inline unsigned int Lib_strtods(const char* a_cstr,std::vector<double>& a_vals,bool a_fixed = false){
  return Lib::strtods(a_cstr,a_vals,a_fixed);
}



#endif
