#
# Manipulate data (here some "OnX_Shape") by using its
# data accessor "type".
#
# All the code is in the file :
#   OnX/examples/Callbacks/Shape.cxx
# and put in the OnXExas DLL.
#
# The "user data class", OnX::Shape, is manipulated
# by the "AccessorManager" system through its
# OnX::ShapeType accessor class.
#
# We load a shapes.onx having a page viewer
# but also a menu bar with a "File/Dump picked"
# to do some operation on the picked (highlighted)
# objects.
#
# Usage :
#   OS> <source osc_vis setup>
#   OS> <source osc_vis python setup>
#   OS> <start Python>
#   >>> import shapes
#

def callback():
  import Slash
  session = Slash.session()
  da = session.accessorManager() # data accessor.  
  da.collect('SceneGraph(Viewer)','highlight==true')  
  da.filter('name')  
  hit = da.handlersIterator()  
  print 'Picked objects :'  
  while 1:
    h = hit.handler()
    if h == None:break
    accessor = h.type()
    obj = h.object(); # void*
    print 'Accessor %s object 0x%lx' % (accessor.name(),obj)
    #print obj
    #s = OnX.smanip_p2s(obj)
    #print s
    hit.next()
  hit.thisown = 1
  del hit

import Slash
session = Slash.create_session()
ui = session.createUI('shapes.onx')

import CoinPython as Inventor
import HEPVis

ui.executeScript('DLD','OnXExas shape_initialize')

da = session.accessorManager() # data accessor.

soPage = ui.findSoPage('Viewer')
soPage.title.setValue('OnX/Shapes')
soPage.titleVisible.setValue(0)
soRegion = soPage.currentRegion()
soRegion.color.setValue(Inventor.SbColor(0.8,0.8,0.8))
del soRegion
del soPage

session.setParameter('modeling.modeling','solid')

session.setParameter('modeling.transparency','0.4')
session.setColor('red')
da.collect('OnX_Shape','type=="Cube"')
da.visualize('Viewer')

session.setParameter('modeling.transparency','0')
session.setColor('yellow')
da.collect('OnX_Shape','type=="Hole"')
da.visualize('Viewer')

session.setColor('blue')
session.setParameter('modeling.modeling','texts')
#da.collect('OnX_Shape','type=="Sphere" && size==1')
da.collect('OnX_Shape','type=="Sphere"')
da.visualize('Viewer')

widget = ui.findWidget('Viewer')
if widget != None:
  soViewer = widget.cast_ISoViewer()
  if soViewer != None:
    soViewer.setFeedbackVisibility(1)
  
# so that the "Dump picked" works :
ui.executeScript('DLD','OnX OnX_declare_accessors')

ui.steer()
del session
