
#include <Slash/Core/ISession.h>

//#include <Slash/Tools/UI.h>
#include <Slash/Tools/Inventor.h>
#include <Slash/Tools/IvNet.h>

#include <HEPVis/nodekits/SoPage.h>
#include <HEPVis/nodekits/SoRegion.h>
#include <HEPVis/nodekits/SoDetectorExample.h>

static void create_ui(Slash::Core::ISession& a_session,std::ostream& a_out,const std::string& a_viewer,bool a_ui) {

  Slash::UI::IUI* ui = Slash::find_UI(a_session);     
  if(!ui) {
    a_out << "Slash::UI::IUI not found." << std::endl;
    return;
  }

  SoPage* soPage = Slash::find_SoPage(*ui,a_viewer);
  if(!soPage) return;
  soPage->title.setValue("OnX/Detector");

  SoRegion* soRegion = soPage->currentRegion();
  if(!soRegion) return;

  SoDetectorExample* det = new SoDetectorExample();

  if(!Slash::add_scene(*ui,*soRegion,det,a_viewer,"staticScene")) 
    det->unref();

  if(a_ui) ui->steer();

}

#include <Slash/Core/ISession.h>
#include <Slash/Tools/osc_env.h> //For TTFPATH

#include <iostream>

int main(int argc,char* argv[]) {

  if(!Slash::osc::set_env(std::cout)) {
    std::cout << "create_ui :"
              << " unable to set OSC environment." << std::endl;
    return 1;
  }

  Slash::Core::ISession* session = 
    Slash_create_session(Slash::to(argc,argv));
  if(!session) {
    std::cout << "create_ui :"
              << " session not found." << std::endl;
    return 1;
  }

  Slash::UI::IUI_Manager* uim = Slash::uiManager(*session);
  if(!uim) {
    std::cout << "create_ui :"
              << " UI_Manager not found." << std::endl;
    return 1;
  }

  std::vector<std::string> args;
  uim->create("","detector.onx",args,true);

  create_ui(*session,std::cout,"@current@",true);
  delete session;

  return 0;
}
