
#include <Slash/Core/ISession.h>

//#include <Slash/Tools/UI.h>
#include <Slash/Tools/Inventor.h>
#include <Slash/Tools/IvNet.h>

#include <Lib/smanip.h>

#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoTransform.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoCube.h>

#include <HEPVis/nodekits/SoPage.h>
#include <HEPVis/nodekits/SoRegion.h>

static void mire(Slash::Core::ISession& a_session,std::ostream& a_out,const std::string& a_viewer,bool a_ui) {

  Slash::UI::IUI* ui = Slash::find_UI(a_session);     
  if(!ui) {
    a_out << "Slash::UI::IUI not found." << std::endl;
    return;
  }

  std::vector<std::string> colors;
  colors.push_back("aquamarine");
  colors.push_back("mediumaquamarine");
  colors.push_back("black");
  colors.push_back("blue");
  colors.push_back("cadetblue");
  colors.push_back("cornflowerblue");
  colors.push_back("darkslateblue");
  colors.push_back("lightblue");
  colors.push_back("lightsteelblue");
  colors.push_back("mediumblue");

  colors.push_back("mediumslateblue");
  colors.push_back("midnightblue");
  colors.push_back("navyblue");
  colors.push_back("navy");
  colors.push_back("skyblue");
  colors.push_back("slateblue");
  colors.push_back("steelblue");
  colors.push_back("coral");
  colors.push_back("cyan");
  colors.push_back("firebrick");

  colors.push_back("brown");
  colors.push_back("gold");
  colors.push_back("goldenrod");
  colors.push_back("green");
  colors.push_back("darkgreen");
  colors.push_back("darkolivegreen");
  colors.push_back("forestgreen");
  colors.push_back("limegreen");
  colors.push_back("mediumseagreen");
  colors.push_back("mediumspringgreen");

  colors.push_back("palegreen");
  colors.push_back("seagreen");
  colors.push_back("springgreen");
  colors.push_back("yellowgreen");
  colors.push_back("darkslategrey");
  colors.push_back("dimgrey");
  colors.push_back("lightgrey");
  colors.push_back("grey");
  colors.push_back("khaki");
  colors.push_back("magenta");

  colors.push_back("maroon");
  colors.push_back("orange");
  colors.push_back("orchid");
  colors.push_back("darkorchid");
  colors.push_back("mediumorchid");
  colors.push_back("pink");
  colors.push_back("plum");
  colors.push_back("red");
  colors.push_back("indianred");
  colors.push_back("mediumvioletred");
      
  colors.push_back("orangered");
  colors.push_back("violetred");
  colors.push_back("salmon");
  colors.push_back("sienna");
  colors.push_back("tan");
  colors.push_back("thistle");
  colors.push_back("turquoise");
  colors.push_back("darkturquoise");
  colors.push_back("mediumturquoise");
  colors.push_back("violet");

  colors.push_back("blueviolet");
  colors.push_back("wheat");
  colors.push_back("white");
  colors.push_back("yellow");
  colors.push_back("greenyellow");

  SoPage* soPage = Slash::find_SoPage(*ui,a_viewer);
  if(!soPage) return;
  soPage->title.setValue("OnX/Detector");

  SoRegion* soRegion = soPage->currentRegion();
  if(!soRegion) return;

  SoSeparator* sep = new SoSeparator;

  unsigned int index = 0;

  double y = 0;    
  for(unsigned int irow=0;irow<10;irow++) {
    double x = 0;
    for(unsigned int icol=0;icol<10;icol++) {
      if(index>=colors.size()) break;  

      SoSeparator* sep2 = new SoSeparator;
      sep->addChild(sep2);

      SoTransform* soTrans = new SoTransform;
      soTrans->translation.setValue(SbVec3f(x,y,0));
      sep2->addChild(soTrans);

      double r,g,b;
      Lib::smanip::torgb(colors[index],r,g,b);
      SoMaterial* soMat = new SoMaterial;
      soMat->diffuseColor.setValue(SbVec3f(r,g,b));
      sep2->addChild(soMat);

      SoCube* soCube = new SoCube;
      sep2->addChild(soCube);

      index++;

      x += 5;
    }
    y -= 5;
  }

  if(!Slash::add_scene(*ui,*soRegion,sep,a_viewer,"staticScene")) 
    sep->unref();

  if(a_ui) ui->steer();

}

#include <Slash/Core/ISession.h>
#include <Slash/Tools/osc_env.h> //For TTFPATH

#include <iostream>

int main(int,char*[]) {

  if(!Slash::osc::set_env(std::cout)) {
    std::cout << "create_ui :"
              << " unable to set OSC environment." << std::endl;
    return 1;
  }

  int argc = 1;
  static char* argv[] = {(char*)"-onx_file=mire.onx"};  

  Slash::Core::ISession* session = Slash_create_session(Slash::to(argc,argv));
  if(!session) {
    std::cout << "mire :"
              << " session not found." << std::endl;
    return 1;
  }

  mire(*session,std::cout,"@current@",true);
  delete session;

  return 0;
}
