#ifndef OnX_NetUI_h
#define OnX_NetUI_h 

// Inheritance :
#include <Slash/Core/IManager.h>
#include <Slash/UI/IUI.h>

#include <OnX/Core/CallbackData.h>
#include <Lib/InternetSocket.h>

namespace Slash {namespace Core {class ISession;}}
namespace Slash {namespace Core {class IWriter;}}
namespace Slash {namespace UI {class IScriptManager;}}
namespace Slash {namespace Net {class INetManager;}}

namespace OnX {

class NetUI : public virtual Slash::UI::IUI {
public: //Slash::UI::IUI :
  virtual void* cast(const std::string&) const;
  virtual bool executeScript(const std::string&,const std::string&,const Slash::UI::IUI::Options& = Slash::UI::IUI::Options());
  virtual void put(const std::string&);
  virtual int steer();
  virtual int steerLevel() const;
  virtual void synchronize();
  virtual void exit(int = 0);
  virtual bool lock();
  virtual bool unlock();
  virtual bool show();
  virtual void echo(const std::string&);
  virtual void warn(const std::string&);
  virtual std::string ask(const std::string&);
  virtual void addDispatcher(Slash::UI::IUI::Dispatcher);
  virtual bool removeCallback(const std::string&,
                              const std::string&,
                              int = 0);
  virtual bool setCallback(const std::string&,
                           const std::string&,
                           const std::string&,
                           const std::string&,
                           int = 0);
  virtual std::string callbackValue();
  virtual const Slash::UI::ICallbackData& callbackData();
  virtual const std::string& thisComponent();
  virtual bool parameterValue(const std::string&,std::string&);
  virtual bool setParameter(const std::string&,const std::string&);
  virtual bool executeCallbacks(const std::string&,const std::string&);
  virtual bool load(const std::string&,const std::string&,bool = true,bool = true);
  virtual bool save();
  virtual void deleteChildren(const std::string&);

  virtual Slash::UI::IWidget* findWidget(const std::string&) const;
  virtual Slash::UI::IWidget* findWidgetByAttribute(const std::string& atb,
                                                    const std::string& value,
                                                    bool ud = true,
                                                    bool lr = true) const;
  virtual bool load(Slash::UI::IWidget& parent,const std::string&,bool = true,bool = true);

  virtual Slash::UI::IWidget* createWidget(
                                const Slash::UI::IWidgetClass& aClass,
                                const std::string& aName,
                                Slash::UI::IWidget& aParent,
                                const std::vector<std::string>& aArgs = std::vector<std::string>());

  virtual bool destroyWidget(Slash::UI::IWidget*&);

  virtual Slash::UI::ICyclic* createCyclic(const std::string&,
                                int,const std::string&,const std::string&);
  virtual Slash::UI::ICyclic* findCyclic(const std::string&) const;
  virtual bool removeCyclic(Slash::UI::ICyclic*&);
  virtual bool createComponent(const std::string& aName,
                               const std::string& aClass,
                               const std::string& aParent);
  virtual bool copyComponent(const std::string&,
                             const std::string&,
                             const std::string&);
  virtual std::vector<std::string> findChildren(const std::string&,
                                                const std::string&,
                                                const std::string&) const;
  virtual std::vector<std::string> findParent(const std::string& aComponent,
                                                const std::string& aWhat,
                                                const std::string& aParameter) const;
  virtual bool destroyComponent(const std::string&);
  virtual bool isComponent(const std::string&);
  virtual Slash::Core::ISession& session();
  virtual bool beingDestroyed() const;
  virtual bool finalize();

  virtual bool isValid() const;
  virtual bool setCallbackData(const Slash::UI::ICallbackData&);
  virtual std::string createWidgetName();
  virtual Slash::UI::IWidgetClass* findWidgetClass(const std::string&) const;
  virtual Slash::UI::IWidget* currentWidget() const; 
  virtual bool setCurrentWidget(Slash::UI::IWidget*); 
  virtual bool enableTerminal(const std::string&,const std::string&);
  virtual bool disableTerminal();
  virtual bool redirectAskToTerminal(bool); 
  virtual bool lockScreen(const std::string& passwd,
                          const std::string& interp = "",
                          const std::string& script = ""); 
  virtual bool atat(const std::string&,std::string&) const;

  virtual bool show(const std::string&);
public: //Lib::IAuditor
  virtual bool notify(Slash::Core::IMessage*);
public:
  NetUI(Slash::Core::ISession&,const std::vector<std::string>&);
  virtual ~NetUI();
  Slash::Core::IWriter& printer() const;
private:
  Slash::Core::ISession& fSession;
  Slash::UI::IScriptManager* fScriptManager;
  Slash::Net::INetManager* fNetManager;
  int fVerboseLevel;
  bool fIsValid;
  Lib::InternetSocket fClient;
  Slash::UI::IWidget* fCurrentWidget;
  bool fBeingDestroyed;

  std::string fValue;
  CallbackData fCallbackData;
};

}

#endif

