#ifndef OnX_QTk_h
#define OnX_QTk_h 

// Inheritance :
#include <QtGui/qapplication.h>
#include <QtGui/qsplitter.h>
#include <QtGui/qdialog.h>
#include <QtGui/qmainwindow.h>
#include <QtGui/qevent.h>
#include <QtGui/qcolordialog.h>
#include <QtOpenGL/qgl.h>

class QCallbackData;

// to handle QFileOpenEvent :
class OnX_QApplication : public QApplication {
  Q_OBJECT
public:
  OnX_QApplication(int&,char**);
  virtual ~OnX_QApplication();
public:
  typedef bool(*OpenFileCB)(const QString&,void*);
  void setOpenFileCallback(OpenFileCB,void*);
protected:
  virtual bool event(QEvent*);
private:
  OpenFileCB fCB;
  void* fTag;
};

class OnX_QOpenGLArea : public QGLWidget {
  Q_OBJECT
public:
  OnX_QOpenGLArea(QWidget*);
public:
  virtual ~OnX_QOpenGLArea();
  virtual void initialieGL();
  virtual void resizeGL(int,int);
  virtual void paintGL();
  virtual void mousePressEvent(QMouseEvent*);
  virtual void mouseReleaseEvent(QMouseEvent*);
  virtual void mouseMoveEvent(QMouseEvent*);
signals:
  void paint(QCallbackData&);
  void eventHandler(QCallbackData&);
public:
  bool write_gl2ps(const std::string&,const std::string&); 
};

// To handle splitPosition at startup.
class OnX_QSplitter : public QSplitter {
  Q_OBJECT
public:
  OnX_QSplitter(Qt::Orientation,QWidget*);
  virtual ~OnX_QSplitter();
  void setSplitPosition(int);
  virtual void showEvent(QShowEvent*);
private:
  int fSplitPosition;
};

#if QT_VERSION < 0x040600
class OnX_QColorDialog_Base : public QWidget {
#else
class OnX_QColorDialog_Base : public QColorDialog {
#endif
public:
  OnX_QColorDialog_Base();
  virtual ~OnX_QColorDialog_Base();
};

class OnX_QColorDialog : public OnX_QColorDialog_Base {
  Q_OBJECT
public:
  OnX_QColorDialog();
  virtual ~OnX_QColorDialog();
public:
  std::string value() const;
  void setOkHide(bool);
signals:
  void ok();
private slots:
  void color_selected();
#if QT_VERSION < 0x040600
public:
  void map();
#else
private:
  virtual void done(int);
  virtual void setVisible(bool);
private:
  bool f_ok_hide;
  bool f_do_done;
#endif
private:
  std::string fValue;
};

class OnX_QFileDialog : public QWidget {
  Q_OBJECT
public:
  virtual void map();
public:
  OnX_QFileDialog();
  virtual ~OnX_QFileDialog();
  std::string value() const;
  enum AcceptMode { AcceptOpen, AcceptSave };
  void setAcceptMode(AcceptMode);
  void setFilter(const std::string&);
  void setDirectory(const std::string&);
signals:
  void ok();
private:
  AcceptMode fMode;
  std::string fFilter;
  std::string fDirectory;
  std::string fValue;
};

class OnX_QMenu : public QWidget {
  Q_OBJECT
public:
  OnX_QMenu(QMenu&);
  virtual ~OnX_QMenu();
  QMenu& popupMenu() const;
private:
  QMenu& fPopup;
};

class OnX_QMenuItem : public QWidget {
  Q_OBJECT
public:
  OnX_QMenuItem(QMenu&,const std::string&);
  virtual ~OnX_QMenuItem();
  QMenu& popupMenu() const;
  //int id() const;
  void set(const std::string&);
  void setAccel(const std::string&);
  bool label(std::string&) const;
  bool setLabel(const std::string&);
signals:
  void activated();
public slots:
  void execute(); //the popup activates it.
protected:
  QMenu& fPopup;
  QAction* fAction;
};

class OnX_QToggleMenuItem : public OnX_QMenuItem {
  Q_OBJECT
public:
  OnX_QToggleMenuItem(QMenu&,const std::string&);
  virtual ~OnX_QToggleMenuItem();
public:
  bool isChecked(bool&) const;
  bool setChecked(bool);
};

class OnX_QMenuSeparator : public QWidget {
  Q_OBJECT
public:
  OnX_QMenuSeparator(QMenu&);
  virtual ~OnX_QMenuSeparator();
protected:
  QMenu& fPopup;
  QAction* fAction;
};

class OnX_QMainWindow : public QMainWindow {
  Q_OBJECT
public:
  OnX_QMainWindow(QWidget*,Qt::WFlags);
  virtual ~OnX_QMainWindow();
protected:
  virtual void closeEvent(QCloseEvent*);
signals:
  void close_window(); //can't be close. It clashes with QWidget::close.
};

#endif
