#ifndef Slash_Store_IStorable_h
#define Slash_Store_IStorable_h

#include <string>

namespace Slash {namespace Store { class IConstVisitor;}}
namespace Slash {namespace Store { class IVisitor;}}

namespace Slash {

namespace Store {

class IStorable {
public:
  virtual ~IStorable() {}
public:
  virtual void* cast(const std::string&) const = 0;
public:
  virtual bool visit(IConstVisitor&) const = 0;
  virtual bool read(IVisitor&) = 0;
};

} //Store

} //Slash

#define SLASH_STORE_HEADER \
private:\
  static bool visit(const Slash::Store::IStorable&,\
                         Slash::Store::IConstVisitor&);

#if defined(__linux)

// Python shell (2.3.3) looses the dynamic_cast on the DLL modules !
// Then we stay with our "safe cast".
// Stay in sync with Lib/Cast.h.

#define SLASH_STORE_SOURCE(aClass) \
bool aClass::visit(\
 const Slash::Store::IStorable& aObject\
,Slash::Store::IConstVisitor& aVisitor\
){\
  const aClass* local = (const aClass*)aObject.cast(#aClass);\
  if(!local) {\
    /*fprintf(stderr,"debug : %s::visit cast %ld failed\n",#aClass,local);*/\
    return false;\
  }\
  return local->aClass::visit(aVisitor);\
}

#else

#define SLASH_STORE_SOURCE(aClass) \
bool aClass::visit(\
 const Slash::Store::IStorable& aObject\
,Slash::Store::IConstVisitor& aVisitor\
){\
  const aClass* local = dynamic_cast<const aClass*>(&aObject);\
  if(!local) {\
    /*fprintf(stderr,"debug : %s::visit cast %ld failed\n",#aClass,local);*/\
    return false;\
  }\
  return local->aClass::visit(aVisitor);\
}

#endif

#define SLASH_STORE_BEGIN(aClass) \
  if(!aVisitor.begin(*this,#aClass,aClass::visit)) return false;

#endif
