#ifndef SoSDL_app_h
#define SoSDL_app_h 

// Some standard callbacks.

#include <Inventor/nodes/SoEventCallback.h>
#include <Inventor/events/SoKeyboardEvent.h>

#include <Inventor/SDL/viewers/SoSDL_Viewer.h>

inline void keyboardCB(void*,SoEventCallback* aEventCB){
  if(aEventCB->isHandled()) return;
  const SoKeyboardEvent* event = (SoKeyboardEvent*)aEventCB->getEvent();
  //if(SO_KEY_PRESS_EVENT(event,Q)) //too close of A,D,S,W
  if(SO_KEY_PRESS_EVENT(event,ESCAPE)) {
    SoSDL::toQuit();
    aEventCB->setHandled();

  } else if( SO_KEY_PRESS_EVENT(event,RIGHT_ARROW) ||
             SO_KEY_PRESS_EVENT(event,D)           ){
    SoSDL_Viewer* viewer = SoSDL::getTopLevelWidget()->getViewer();
    if(event->wasShiftDown()) {
      viewer->updateMotion(SoSDL::TRANSLATE_RIGHT);
    } else {
      viewer->updateMotion(SoSDL::ROTATE_RIGHT);
    }
    SoSDL::setViewerRender();
    aEventCB->setHandled();

  } else if( SO_KEY_PRESS_EVENT(event,LEFT_ARROW) ||
             SO_KEY_PRESS_EVENT(event,A)           ){
    SoSDL_Viewer* viewer = SoSDL::getTopLevelWidget()->getViewer();
    if(event->wasShiftDown()) {
      viewer->updateMotion(SoSDL::TRANSLATE_LEFT);
    } else {
      viewer->updateMotion(SoSDL::ROTATE_LEFT);
    }
    SoSDL::setViewerRender();
    aEventCB->setHandled();

  } else if( SO_KEY_PRESS_EVENT(event,UP_ARROW) ||
             SO_KEY_PRESS_EVENT(event,W)           ){
    SoSDL_Viewer* viewer = SoSDL::getTopLevelWidget()->getViewer();
    if(event->wasShiftDown()) {
      viewer->updateMotion(SoSDL::UP);
    } else {
      viewer->updateMotion(SoSDL::FORWARD);
    }
    SoSDL::setViewerRender();
    aEventCB->setHandled();

  } else if( SO_KEY_PRESS_EVENT(event,DOWN_ARROW) ||
             SO_KEY_PRESS_EVENT(event,S)           ){
    SoSDL_Viewer* viewer = SoSDL::getTopLevelWidget()->getViewer();
    if(event->wasShiftDown()) {
      viewer->updateMotion(SoSDL::DOWN);
    } else {
      viewer->updateMotion(SoSDL::BACKWARD);
    }
    SoSDL::setViewerRender();
    aEventCB->setHandled();
  }
}

#include <Inventor/SDL/SoJoyAxisEvent.h>
#include <Inventor/SDL/SoJoyButtonEvent.h>

class JoyTag {
public:
  JoyTag()
  :fMoveWhat(SoSDL::ROTATE_RIGHT)
  ,fRotate(true)
  ,fForBack(true)
  ,fMoveTimerInterval(30) //msec
  ,fMoveTimerID(0)
  {}
  virtual ~JoyTag(){
    if(fMoveTimerID) SDL_RemoveTimer(fMoveTimerID);
  }
public:
  void stopTimer() {
    if(fMoveTimerID) SDL_RemoveTimer(fMoveTimerID);
    fMoveTimerID = 0;
  }
  void startTimer() {
    if(fMoveTimerID) SDL_RemoveTimer(fMoveTimerID);
    fMoveTimerID = SDL_AddTimer(fMoveTimerInterval,moveTimerCB,this);
  }
  static Uint32 moveTimerCB(Uint32 aInterval,void* aTag){
    JoyTag* tag = (JoyTag*)aTag;
    //printf("debug : moveTimerCB : %d\n",tag->fMoveWhat);
    SoSDL_Viewer* viewer = SoSDL::getTopLevelWidget()->getViewer();
    viewer->updateMotion(tag->fMoveWhat);
    return aInterval;
  }
public:
  SoSDL::Move fMoveWhat;
  bool fRotate;
  bool fForBack;
private:
  Uint32 fMoveTimerInterval; //msec
  SDL_TimerID fMoveTimerID;
};

inline void joyAxisCB(void* aTag,SoEventCallback* aEventCB){
  if(aEventCB->isHandled()) return;
  const SoJoyAxisEvent* event = (SoJoyAxisEvent*)aEventCB->getEvent();
  JoyTag* tag = (JoyTag*)aTag;
  //printf("debug : joy_CB\n");
  if(event->getState()==SoJoyAxisEvent::STOP) {
    //printf("debug : joy_CB : STOP\n");
    tag->stopTimer();
    aEventCB->setHandled();
  } else if(event->getState()==SoJoyAxisEvent::RIGHT) {
    //printf("debug : joy_CB : RIGHT\n");
    tag->fMoveWhat = 
      (tag->fRotate?SoSDL::ROTATE_RIGHT:SoSDL::TRANSLATE_RIGHT);
    tag->startTimer();
    aEventCB->setHandled();
  } else if(event->getState()==SoJoyAxisEvent::LEFT) {
    //printf("debug : joy_CB : LEFT\n");
    tag->fMoveWhat = 
      (tag->fRotate?SoSDL::ROTATE_LEFT:SoSDL::TRANSLATE_LEFT);
    tag->startTimer();
    aEventCB->setHandled();
  } else if(event->getState()==SoJoyAxisEvent::DOWN) {
    //printf("debug : joy_CB : DOWN\n");
    tag->fMoveWhat = (tag->fForBack?SoSDL::BACKWARD:SoSDL::DOWN);
    tag->startTimer();
    aEventCB->setHandled();
  } else if(event->getState()==SoJoyAxisEvent::UP) {
    //printf("debug : joy_CB : UP\n");
    tag->fMoveWhat = (tag->fForBack?SoSDL::FORWARD:SoSDL::UP);
    tag->startTimer();
    aEventCB->setHandled();
  }
}

inline void joyButtonCB(void* aTag,SoEventCallback* aEventCB){
  if(aEventCB->isHandled()) return;
  const SoJoyButtonEvent* event = (SoJoyButtonEvent*)aEventCB->getEvent();
  JoyTag* tag = (JoyTag*)aTag;
  //printf("debug : joyButtonCB\n");
  if(event->getState()==SoJoyButtonEvent::DOWN) {
    // on a Logitech Precision the button num = jbutton.button+1
    if(event->getButton()==0) {
      tag->fRotate = tag->fRotate?false:true;
      aEventCB->setHandled();

    } else if(event->getButton()==1) {
      tag->fForBack = tag->fForBack?false:true;
      aEventCB->setHandled();

    } else if( (event->getButton()==4) ||
               (event->getButton()==5) ){
      //printf("debug : joyButtonCB : 4,5\n");
      SoSDL::emulateKeyDown(SDLK_RETURN);
      aEventCB->setHandled();

    } else if( (event->getButton()==6) ||
               (event->getButton()==7) ){
      //printf("debug : joyButtonCB : 6,7\n");
      SoSDL::emulateKeyDown(SDLK_r);
      aEventCB->setHandled();

    } else if(event->getButton()==8) {
      SoSDL::toQuit();
      aEventCB->setHandled();

    } else if(event->getButton()==9) {
      SoSDL::emulateKeyDown(SDLK_m);
      aEventCB->setHandled();
    }

  } else if(event->getState()==SoJoyButtonEvent::UP) {
    // on a Logitech Precision the button num = jbutton.button+1
    if( (event->getButton()==4) ||
               (event->getButton()==5) ){
      //printf("debug : joyButtonCB : 4,5\n");
      SoSDL::emulateKeyUp(SDLK_RETURN);
      aEventCB->setHandled();

    } else if( (event->getButton()==6) ||
               (event->getButton()==7) ){
      //printf("debug : joyButtonCB : 6,7\n");
      SoSDL::emulateKeyUp(SDLK_r);
      aEventCB->setHandled();
    }
  }
}

#include <Inventor/nodes/SoSeparator.h>

inline void add_CBs(SoSeparator* aSep,JoyTag* aTag) {
  SoEventCallback* eventCallback = new SoEventCallback;
  eventCallback->addEventCallback(SoKeyboardEvent::getClassTypeId(),
                                  keyboardCB,0);
  eventCallback->addEventCallback(SoJoyButtonEvent::getClassTypeId(),
                                  joyButtonCB,aTag);
  eventCallback->addEventCallback(SoJoyAxisEvent::getClassTypeId(),
                                  joyAxisCB,aTag);
  aSep->addChild(eventCallback);
}

#endif
